@@ -33,7 +33,9 @@ import com.smartgwt.client.widgets.tile.*;
 import com.smartgwt.client.widgets.tile.events.*;
 import com.smartgwt.client.widgets.grid.*;
 import com.smartgwt.client.widgets.grid.events.*;
+import com.smartgwt.client.widgets.chart.*;
 import com.smartgwt.client.widgets.layout.*;
+import com.smartgwt.client.widgets.layout.events.*;
 import com.smartgwt.client.widgets.menu.*;
 import com.smartgwt.client.widgets.tab.*;
 import com.smartgwt.client.widgets.toolbar.*;
@@ -148,6 +150,7 @@ public class HTMLFlow extends Canvas  implements com.smartgwt.client.widgets.eve
      * The contents of a canvas or label widget. Any HTML string is acceptable.
      *
      * @param contents contents Default value is "&nbsp;"
+     * @see com.smartgwt.client.widgets.HTMLFlow#setDynamicContents
      */
     public void setContents(String contents) {
         setAttribute("contents", contents, true);
@@ -158,6 +161,7 @@ public class HTMLFlow extends Canvas  implements com.smartgwt.client.widgets.eve
      *
      *
      * @return String
+     * @see com.smartgwt.client.widgets.HTMLFlow#getDynamicContents
      */
     public String getContents()  {
         return getAttributeAsString("contents");
@@ -211,6 +215,8 @@ public class HTMLFlow extends Canvas  implements com.smartgwt.client.widgets.eve
      * <p><b>Note : </b> This is an advanced setting</p>
      *
      * @param dynamicContents dynamicContents Default value is false
+     * @see com.smartgwt.client.widgets.HTMLFlow#setContents
+     * @see com.smartgwt.client.widgets.Canvas#setDynamicContentsVars
      * 
      */
     public void setDynamicContents(Boolean dynamicContents) {
@@ -241,6 +247,8 @@ public class HTMLFlow extends Canvas  implements com.smartgwt.client.widgets.eve
      *
      *
      * @return Boolean
+     * @see com.smartgwt.client.widgets.HTMLFlow#getContents
+     * @see com.smartgwt.client.widgets.Canvas#getDynamicContentsVars
      * 
      */
     public Boolean getDynamicContents()  {
@@ -295,24 +303,26 @@ public class HTMLFlow extends Canvas  implements com.smartgwt.client.widgets.eve
     }
 
     /**
-     * HTML to show while content is being fetched, active only if the <code>contentsURL</code> property has been set. <P> The
-     * loading message will show both during the initial load of content, and during reload if the contents are reloaded or the
-     * contentsURL changed.  For a first-time only loading message, initialize the <code>contents</code> property instead.<br>
-     * Note: the <code>loadingMessage</code> is never displayed when loading complete web pages  rather than HTML fragments
-     * (see {@link com.smartgwt.client.widgets.HTMLFlow#getContentsType contentsType}).
+     * HTML to show while content is being fetched, active only if the <code>contentsURL</code> property has been set. Use
+     * <code>"\${loadingImage}"</code> to include {@link com.smartgwt.client.widgets.Canvas#loadingImageSrc a loading image}.
+     * <P> The loading message will show both during the initial load of content, and during reload if the contents are
+     * reloaded or the contentsURL changed.  For a first-time only loading message, initialize the <code>contents</code>
+     * property instead.<br> Note: the <code>loadingMessage</code> is never displayed when loading complete web pages  rather
+     * than HTML fragments (see {@link com.smartgwt.client.widgets.HTMLFlow#getContentsType contentsType}).
      *
-     * @param loadingMessage loadingMessage Default value is null
+     * @param loadingMessage loadingMessage Default value is "&amp;nbsp;\${loadingImage}"
      */
     public void setLoadingMessage(String loadingMessage) {
         setAttribute("loadingMessage", loadingMessage, true);
     }
 
     /**
-     * HTML to show while content is being fetched, active only if the <code>contentsURL</code> property has been set. <P> The
-     * loading message will show both during the initial load of content, and during reload if the contents are reloaded or the
-     * contentsURL changed.  For a first-time only loading message, initialize the <code>contents</code> property instead.<br>
-     * Note: the <code>loadingMessage</code> is never displayed when loading complete web pages  rather than HTML fragments
-     * (see {@link com.smartgwt.client.widgets.HTMLFlow#getContentsType contentsType}).
+     * HTML to show while content is being fetched, active only if the <code>contentsURL</code> property has been set. Use
+     * <code>"\${loadingImage}"</code> to include {@link com.smartgwt.client.widgets.Canvas#loadingImageSrc a loading image}.
+     * <P> The loading message will show both during the initial load of content, and during reload if the contents are
+     * reloaded or the contentsURL changed.  For a first-time only loading message, initialize the <code>contents</code>
+     * property instead.<br> Note: the <code>loadingMessage</code> is never displayed when loading complete web pages  rather
+     * than HTML fragments (see {@link com.smartgwt.client.widgets.HTMLFlow#getContentsType contentsType}).
      *
      *
      * @return String
