@@ -33,7 +33,9 @@ import com.smartgwt.client.widgets.tile.*;
 import com.smartgwt.client.widgets.tile.events.*;
 import com.smartgwt.client.widgets.grid.*;
 import com.smartgwt.client.widgets.grid.events.*;
+import com.smartgwt.client.widgets.chart.*;
 import com.smartgwt.client.widgets.layout.*;
+import com.smartgwt.client.widgets.layout.events.*;
 import com.smartgwt.client.widgets.menu.*;
 import com.smartgwt.client.widgets.tab.*;
 import com.smartgwt.client.widgets.toolbar.*;
@@ -119,6 +121,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * children for a node.
      *
      * @param childrenProperty childrenProperty Default value is "children"
+     * @see com.smartgwt.client.widgets.tree.Tree#setModelType
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_databinding_children_arrays" target="examples">Children Arrays Example</a>
      */
     public void setChildrenProperty(String childrenProperty) {
@@ -131,6 +134,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return String
+     * @see com.smartgwt.client.widgets.tree.Tree#getModelType
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_databinding_children_arrays" target="examples">Children Arrays Example</a>
      */
     public String getChildrenProperty()  {
@@ -144,6 +148,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      * @param defaultIsFolder defaultIsFolder Default value is null
      * @throws IllegalStateException this property cannot be changed after the underlying component has been created
+     * @see com.smartgwt.client.widgets.tree.TreeGrid#setLoadDataOnDemand
      */
     public void setDefaultIsFolder(Boolean defaultIsFolder)  throws IllegalStateException {
         setAttribute("defaultIsFolder", defaultIsFolder, false);
@@ -156,6 +161,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return Boolean
+     * @see com.smartgwt.client.widgets.tree.TreeGrid#getLoadDataOnDemand
      */
     public Boolean getDefaultIsFolder()  {
         return getAttributeAsBoolean("defaultIsFolder");
@@ -217,6 +223,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      * @param idField idField Default value is "id"
      * @throws IllegalStateException this property cannot be changed after the underlying component has been created
+     * @see com.smartgwt.client.widgets.tree.TreeNode#setId
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public void setIdField(String idField)  throws IllegalStateException {
@@ -230,6 +237,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return String
+     * @see com.smartgwt.client.widgets.tree.TreeNode#getId
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public String getIdField()  {
@@ -241,6 +249,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * com.smartgwt.client.widgets.tree.TreeNode#getIsFolder isFolder}.
      *
      * @param isFolderProperty isFolderProperty Default value is "isFolder"
+     * @see com.smartgwt.client.widgets.tree.TreeNode#setIsFolder
      */
     public void setIsFolderProperty(String isFolderProperty) {
         setAttribute("isFolderProperty", isFolderProperty, true);
@@ -252,6 +261,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return String
+     * @see com.smartgwt.client.widgets.tree.TreeNode#getIsFolder
      */
     public String getIsFolderProperty()  {
         return getAttributeAsString("isFolderProperty");
@@ -266,6 +276,8 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * <p><b>Note : </b> This is an advanced setting</p>
      *
      * @param modelType modelType Default value is "children"
+     * @see com.smartgwt.client.widgets.tree.Tree#setData
+     * @see com.smartgwt.client.widgets.tree.Tree#setRoot
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public void setModelType(TreeModelType modelType) {
@@ -281,6 +293,8 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return TreeModelType
+     * @see com.smartgwt.client.widgets.tree.Tree#getData
+     * @see com.smartgwt.client.widgets.tree.Tree#getRoot
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public TreeModelType getModelType()  {
@@ -293,6 +307,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * system.  Default value is "name".  See {@link com.smartgwt.client.widgets.tree.TreeNode#getName name} for usage.
      *
      * @param nameProperty nameProperty Default value is "name"
+     * @see com.smartgwt.client.widgets.tree.TreeNode#setName
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public void setNameProperty(String nameProperty) {
@@ -306,6 +321,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return String
+     * @see com.smartgwt.client.widgets.tree.TreeNode#getName
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public String getNameProperty()  {
@@ -350,6 +366,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      * @param parentIdField parentIdField Default value is "parentId"
      * @throws IllegalStateException this property cannot be changed after the underlying component has been created
+     * @see com.smartgwt.client.widgets.tree.TreeNode#setParentId
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public void setParentIdField(String parentIdField)  throws IllegalStateException {
@@ -363,6 +380,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return String
+     * @see com.smartgwt.client.widgets.tree.TreeNode#getParentId
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_appearance_node_titles" target="examples">Node Titles Example</a>
      */
     public String getParentIdField()  {
@@ -389,6 +407,8 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * <p><b>Note : </b> This is an advanced setting</p>
      *
      * @param pathDelim pathDelim Default value is "/"
+     * @see com.smartgwt.client.widgets.tree.Tree#setNameProperty
+     * @see com.smartgwt.client.widgets.tree.Tree#find
      */
     public void setPathDelim(String pathDelim) {
         setAttribute("pathDelim", pathDelim, true);
@@ -414,6 +434,8 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return String
+     * @see com.smartgwt.client.widgets.tree.Tree#getNameProperty
+     * @see com.smartgwt.client.widgets.tree.Tree#find
      */
     public String getPathDelim()  {
         return getAttributeAsString("pathDelim");
@@ -466,6 +488,8 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * Set the root node of the tree.
      *
      * @param root new root node. Default value is null
+     * @see com.smartgwt.client.widgets.tree.Tree#setModelType
+     * @see com.smartgwt.client.widgets.tree.Tree#setRoot
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_databinding_children_arrays" target="examples">Children Arrays Example</a>
      */
     public void setRoot(TreeNode root) {
@@ -491,6 +515,8 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return Returns the root node of the tree.
+     * @see com.smartgwt.client.widgets.tree.Tree#getModelType
+     * @see com.smartgwt.client.widgets.tree.Tree#setRoot
      * @see <a href="http://www.smartclient.com/smartgwt/showcase/#tree_databinding_children_arrays" target="examples">Children Arrays Example</a>
      */
     public TreeNode getRoot()  {
@@ -502,6 +528,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * regardless of whether the node is a leaf or folder?
      *
      * @param separateFolders separateFolders Default value is false
+     * @see com.smartgwt.client.widgets.tree.Tree#setSortFoldersBeforeLeaves
      */
     public void setSeparateFolders(Boolean separateFolders) {
         setAttribute("separateFolders", separateFolders, true);
@@ -513,6 +540,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      *
      *
      * @return Boolean
+     * @see com.smartgwt.client.widgets.tree.Tree#getSortFoldersBeforeLeaves
      */
     public Boolean getSeparateFolders()  {
         return getAttributeAsBoolean("separateFolders");
@@ -665,6 +693,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * Returns the number of items in the current open list.
      *
      * @return number of items in open list
+     * @see com.smartgwt.client.widgets.tree.Tree#getOpenList
      */
     public native int getLength() /*-{
         var self = this.@com.smartgwt.client.core.BaseClass::getOrCreateJsObj()();
@@ -718,6 +747,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * @param node node in question
      *
      * @return path to the node's parent
+     * @see com.smartgwt.client.widgets.tree.Tree#getPath
      */
     public native String getParentPath(TreeNode node) /*-{
         var self = this.@com.smartgwt.client.core.BaseClass::getOrCreateJsObj()();
@@ -734,6 +764,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * @param node node in question
      *
      * @return path to the node
+     * @see com.smartgwt.client.widgets.tree.Tree#getParentPath
      */
     public native String getPath(TreeNode node) /*-{
         var self = this.@com.smartgwt.client.core.BaseClass::getOrCreateJsObj()();
@@ -753,6 +784,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
      * @param node node for which the title is being requested
      *
      * @return title to display
+     * @see com.smartgwt.client.widgets.tree.TreeGrid#getNodeTitle
      */
     public native String getTitle(TreeNode node) /*-{
         var self = this.@com.smartgwt.client.core.BaseClass::getOrCreateJsObj()();
@@ -1630,7 +1662,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
 
     //getters
 
-    protected String getAttribute(String property) {
+    public String getAttribute(String property) {
         if(isCreated()) {
             return JSOHelper.getAttribute(getJsObj(), property);
         } else {
@@ -1638,7 +1670,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected String getAttributeAsString(String property) {
+    public String getAttributeAsString(String property) {
         if(isCreated()) {
             return JSOHelper.getAttribute(getJsObj(), property);
         } else {
@@ -1646,7 +1678,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected Date getAttributeAsDate(String property) {
+    public Date getAttributeAsDate(String property) {
         if(isCreated()) {
             return JSOHelper.getAttributeAsDate(getJsObj(), property);
         } else {
@@ -1654,7 +1686,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected Integer getAttributeAsInt(String property) {
+    public Integer getAttributeAsInt(String property) {
         if(isCreated()) {
             return JSOHelper.getAttributeAsInt(getJsObj(), property);
         } else {
@@ -1662,7 +1694,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected Double getAttributeAsDouble(String property) {
+    public Double getAttributeAsDouble(String property) {
         if(isCreated()) {
             return JSOHelper.getAttributeAsDouble(getJsObj(), property);
         } else {
@@ -1670,7 +1702,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected Element getAttributeAsElement(String property) {
+    public Element getAttributeAsElement(String property) {
         if(isCreated()) {
             return JSOHelper.getAttributeAsElement(getJsObj(), property);
         } else {
@@ -1678,7 +1710,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected JavaScriptObject getAttributeAsJavaScriptObject(String property) {
+    public JavaScriptObject getAttributeAsJavaScriptObject(String property) {
         if(isCreated()) {
             return JSOHelper.getAttributeAsJavaScriptObject(getJsObj(), property);
         } else {
@@ -1686,7 +1718,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected Float getAttributeAsFloat(String property) {
+    public Float getAttributeAsFloat(String property) {
         if(isCreated()) {
             return JSOHelper.getAttributeAsFloat(getJsObj(), property);
         } else {
@@ -1694,7 +1726,7 @@ public class Tree extends BaseClass  implements com.smartgwt.client.widgets.tree
         }
     }
 
-    protected Boolean getAttributeAsBoolean(String property) {
+    public Boolean getAttributeAsBoolean(String property) {
         if(isCreated()) {
             return JSOHelper.getAttributeAsBoolean(getJsObj(), property);
         } else {
