@@ -191,6 +191,24 @@ public class CastNormalizer {
       recordCast(x.getTestType(), x.getExpr());
     }
 
+    private boolean canTriviallyCastJsoSemantics(JReferenceType type,
+        JReferenceType qType) {
+      type = type.getUnderlyingType();
+      qType = qType.getUnderlyingType();
+
+      if (type instanceof JArrayType && qType instanceof JArrayType) {
+        JArrayType aType = (JArrayType) type;
+        JArrayType aqType = (JArrayType) qType;
+        return program.typeOracle.canTriviallyCast(type, qType)
+            || (program.isJavaScriptObject(aType.getLeafType())
+                && program.isJavaScriptObject(aqType.getLeafType()));
+      }
+
+      return program.typeOracle.canTriviallyCast(type, qType)
+            || (program.isJavaScriptObject(type)
+                 && program.isJavaScriptObject(qType));
+    }
+    
     /**
      * Create the data for JSON table to capture the mapping from a class to its
      * query types.
@@ -199,12 +217,14 @@ public class CastNormalizer {
       if (type == null || alreadyRan.contains(type)) {
         return;
       }
-      assert (type == program.getRunTimeType(type));
+      assert (type == type.getUnderlyingType());
 
       alreadyRan.add(type);
 
       // Visit super type.
-      computeSourceType(type.getSuperClass());
+      if (type instanceof JClassType) {
+        computeSourceType(((JClassType) type).getSuperClass());
+      }
 
       if (!program.typeOracle.isInstantiatedType(type)
           || program.isJavaScriptObject(type)) {
@@ -219,11 +239,16 @@ public class CastNormalizer {
       for (JReferenceType qType : queriedTypes.keySet()) {
 
         Set<JReferenceType> querySet = queriedTypes.get(qType);
-        if (program.typeOracle.canTriviallyCast(type, qType)) {
+        /**
+         * Handles JSO[] -> JSO[] case now that canCastTrivially doesn't deal
+         * with JSO cross-casts anymore.
+         */
+        if (canTriviallyCastJsoSemantics(type, qType)) {
 
           for (JReferenceType argType : querySet) {
 
-            if (program.typeOracle.canTriviallyCast(type, argType)) {
+            if (canTriviallyCastJsoSemantics(type, argType) ||
+                program.isJavaScriptObject(qType)) {
               if (yesSet == null) {
                 yesSet = new HashSet<JReferenceType>();
               }
@@ -243,8 +268,7 @@ public class CastNormalizer {
       }
 
       // Create a sparse lookup object.
-      SourceInfo sourceInfo = program.createSourceInfoSynthetic(
-          AssignTypeCastabilityVisitor.class, "typeinfo lookup");
+      SourceInfo sourceInfo = program.createSourceInfoSynthetic(AssignTypeCastabilityVisitor.class);
       JsonObject jsonObject = new JsonObject(sourceInfo,
           program.getJavaScriptObject());
       // Start at 1; 0 is Object and always true.
@@ -273,9 +297,9 @@ public class CastNormalizer {
 
     private void recordCast(JType targetType, JExpression rhs) {
       if (targetType instanceof JReferenceType) {
-        targetType = program.getRunTimeType((JReferenceType) targetType);
+        targetType = ((JReferenceType) targetType).getUnderlyingType();
         // unconditional cast b/c it would've been a semantic error earlier
-        JReferenceType rhsType = program.getRunTimeType((JReferenceType) rhs.getType());
+        JReferenceType rhsType = ((JReferenceType) rhs.getType()).getUnderlyingType();
         // don't record a type for trivial casts that won't generate code
         if (program.typeOracle.canTriviallyCast(rhsType,
             (JReferenceType) targetType)) {
@@ -293,8 +317,8 @@ public class CastNormalizer {
 
     private void recordCastInternal(JReferenceType toType,
         JReferenceType rhsType) {
-      toType = program.getRunTimeType(toType);
-      rhsType = program.getRunTimeType(rhsType);
+      toType = toType.getUnderlyingType();
+      rhsType = rhsType.getUnderlyingType();
       Set<JReferenceType> querySet = queriedTypes.get(toType);
       if (querySet == null) {
         queryIds.put(toType, nextQueryId++);
@@ -419,15 +443,18 @@ public class CastNormalizer {
         replaceExpr = call;
       } else if (toType instanceof JReferenceType) {
         JExpression curExpr = expr;
-        JReferenceType refType = program.getRunTimeType((JReferenceType) toType);
+        JReferenceType refType = ((JReferenceType) toType).getUnderlyingType();
         JReferenceType argType = (JReferenceType) expr.getType();
-        if (program.typeOracle.canTriviallyCast(argType, refType)) {
+        if (program.typeOracle.canTriviallyCast(argType, refType)
+            || (program.typeOracle.isEffectivelyJavaScriptObject(argType)
+            &&  program.typeOracle.isEffectivelyJavaScriptObject(refType))) {
           // just remove the cast
           replaceExpr = curExpr;
         } else {
 
           JMethod method;
-          boolean isJsoCast = program.isJavaScriptObject(refType);
+          boolean isJsoCast =
+              program.typeOracle.isEffectivelyJavaScriptObject(refType);
           if (isJsoCast) {
             // A cast to a concrete JSO subtype
             method = program.getIndexedMethod("Cast.dynamicCastJso");
@@ -535,8 +562,11 @@ public class CastNormalizer {
       JReferenceType argType = (JReferenceType) x.getExpr().getType();
       JReferenceType toType = x.getTestType();
       // Only tests on run-time types are supported
-      assert (toType == program.getRunTimeType(toType));
-      if (program.typeOracle.canTriviallyCast(argType, toType)) {
+      assert (toType == toType.getUnderlyingType());
+      if (program.typeOracle.canTriviallyCast(argType, toType)
+          // don't depend on type-tightener having run
+          || (program.typeOracle.isEffectivelyJavaScriptObject(argType)
+          && program.typeOracle.isEffectivelyJavaScriptObject(toType))) {
         // trivially true if non-null; replace with a null test
         JNullLiteral nullLit = program.getLiteralNull();
         JBinaryOperation eq = new JBinaryOperation(x.getSourceInfo(),
@@ -546,9 +576,9 @@ public class CastNormalizer {
       } else {
         JMethod method;
         boolean isJsoCast = false;
-        if (program.typeOracle.getSingleJsoImpl(toType) != null) {
+        if (program.typeOracle.isDualJsoInterface(toType)) {
           method = program.getIndexedMethod("Cast.instanceOfOrJso");
-        } else if (program.isJavaScriptObject(toType)) {
+        } else if (program.typeOracle.isEffectivelyJavaScriptObject(toType)) {
           isJsoCast = true;
           method = program.getIndexedMethod("Cast.instanceOfJso");
         } else {
