@@ -15,7 +15,6 @@
  */
 package com.google.gwt.dev.js;
 
-import com.google.gwt.dev.jjs.HasSourceInfo;
 import com.google.gwt.dev.jjs.SourceInfo;
 import com.google.gwt.dev.jjs.impl.OptimizerStats;
 import com.google.gwt.dev.js.ast.CanBooleanEval;
@@ -28,6 +27,7 @@ import com.google.gwt.dev.js.ast.JsConditional;
 import com.google.gwt.dev.js.ast.JsContext;
 import com.google.gwt.dev.js.ast.JsContinue;
 import com.google.gwt.dev.js.ast.JsDoWhile;
+import com.google.gwt.dev.js.ast.JsEmpty;
 import com.google.gwt.dev.js.ast.JsExprStmt;
 import com.google.gwt.dev.js.ast.JsExpression;
 import com.google.gwt.dev.js.ast.JsFor;
@@ -76,12 +76,12 @@ public class JsStaticEval {
     private boolean hasBreakContinueStatements = false;
 
     @Override
-    public void endVisit(JsBreak x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsBreak x, JsContext ctx) {
       hasBreakContinueStatements = true;
     }
 
     @Override
-    public void endVisit(JsContinue x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsContinue x, JsContext ctx) {
       hasBreakContinueStatements = true;
     }
 
@@ -109,7 +109,7 @@ public class JsStaticEval {
     }
 
     @Override
-    public void endVisit(JsExprStmt x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsExprStmt x, JsContext ctx) {
       JsFunction func = isFunctionDecl(x);
       if (func != null) {
         mustExec.add(x);
@@ -117,13 +117,11 @@ public class JsStaticEval {
     }
 
     @Override
-    public void endVisit(JsVars x, JsContext<JsStatement> ctx) {
-      JsVars strippedVars = new JsVars(x.getSourceInfo().makeChild(
-          MustExecVisitor.class, "Simplified execution"));
+    public void endVisit(JsVars x, JsContext ctx) {
+      JsVars strippedVars = new JsVars(x.getSourceInfo());
       boolean mustReplace = false;
       for (JsVar var : x) {
-        JsVar strippedVar = new JsVar(var.getSourceInfo().makeChild(
-            MustExecVisitor.class, "Simplified execution"), var.getName());
+        JsVar strippedVar = new JsVar(var.getSourceInfo(), var.getName());
         strippedVars.add(strippedVar);
         if (var.getInitExpr() != null) {
           mustReplace = true;
@@ -141,7 +139,7 @@ public class JsStaticEval {
     }
 
     @Override
-    public boolean visit(JsFunction x, JsContext<JsExpression> ctx) {
+    public boolean visit(JsFunction x, JsContext ctx) {
       // Don't dive into nested functions.
       return false;
     }
@@ -164,7 +162,7 @@ public class JsStaticEval {
     private Map<JsExpression, Boolean> coercesToStringMap = new IdentityHashMap<JsExpression, Boolean>();
 
     @Override
-    public void endVisit(JsBinaryOperation x, JsContext<JsExpression> ctx) {
+    public void endVisit(JsBinaryOperation x, JsContext ctx) {
       JsBinaryOperator op = x.getOperator();
       JsExpression arg1 = x.getArg1();
       JsExpression arg2 = x.getArg2();
@@ -191,7 +189,7 @@ public class JsStaticEval {
      * Prune dead statements and empty blocks.
      */
     @Override
-    public void endVisit(JsBlock x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsBlock x, JsContext ctx) {
       /*
        * Remove any dead statements after an abrupt change in code flow and
        * promote safe statements within nested blocks to this block.
@@ -235,7 +233,7 @@ public class JsStaticEval {
     }
 
     @Override
-    public void endVisit(JsConditional x, JsContext<JsExpression> ctx) {
+    public void endVisit(JsConditional x, JsContext ctx) {
       evalBooleanContext.remove(x.getTestExpression());
 
       JsExpression condExpr = x.getTestExpression();
@@ -244,15 +242,13 @@ public class JsStaticEval {
       if (condExpr instanceof CanBooleanEval) {
         CanBooleanEval condEval = (CanBooleanEval) condExpr;
         if (condEval.isBooleanTrue()) {
-          JsBinaryOperation binOp = new JsBinaryOperation(makeSourceInfo(x,
-              "Simplified always-true condition"), JsBinaryOperator.AND,
-              condExpr, thenExpr);
+          JsBinaryOperation binOp = new JsBinaryOperation(x.getSourceInfo(),
+              JsBinaryOperator.AND, condExpr, thenExpr);
           ctx.replaceMe(accept(binOp));
         } else if (condEval.isBooleanFalse()) {
           // e.g. (false() ? then : else) -> false() || else
-          JsBinaryOperation binOp = new JsBinaryOperation(makeSourceInfo(x,
-              "Simplified always-false condition"), JsBinaryOperator.OR,
-              condExpr, elseExpr);
+          JsBinaryOperation binOp = new JsBinaryOperation(x.getSourceInfo(),
+              JsBinaryOperator.OR, condExpr, elseExpr);
           ctx.replaceMe(accept(binOp));
         }
       }
@@ -262,7 +258,7 @@ public class JsStaticEval {
      * Convert do { } while (false); into a block.
      */
     @Override
-    public void endVisit(JsDoWhile x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsDoWhile x, JsContext ctx) {
       evalBooleanContext.remove(x.getCondition());
 
       JsExpression expr = x.getCondition();
@@ -275,8 +271,7 @@ public class JsStaticEval {
           FindBreakContinueStatementsVisitor visitor = new FindBreakContinueStatementsVisitor();
           visitor.accept(x.getBody());
           if (!visitor.hasBreakContinueStatements()) {
-            JsBlock block = new JsBlock(makeSourceInfo(x,
-                "Simplified always-false condition"));
+            JsBlock block = new JsBlock(x.getSourceInfo());
             block.getStatements().add(x.getBody());
             block.getStatements().add(expr.makeStmt());
             ctx.replaceMe(accept(block));
@@ -286,12 +281,12 @@ public class JsStaticEval {
     }
 
     @Override
-    public void endVisit(JsExprStmt x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsExprStmt x, JsContext ctx) {
       if (!x.getExpression().hasSideEffects()) {
         if (ctx.canRemove()) {
           ctx.removeMe();
         } else {
-          ctx.replaceMe(program.getEmptyStmt());
+          ctx.replaceMe(new JsEmpty(x.getSourceInfo()));
         }
       }
     }
@@ -300,7 +295,7 @@ public class JsStaticEval {
      * Prune for (X; false(); Y) statements, make sure X and false() are run.
      */
     @Override
-    public void endVisit(JsFor x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsFor x, JsContext ctx) {
       evalBooleanContext.remove(x.getCondition());
 
       JsExpression expr = x.getCondition();
@@ -309,8 +304,7 @@ public class JsStaticEval {
 
         // If false, replace with initializers and condition.
         if (cond.isBooleanFalse()) {
-          JsBlock block = new JsBlock(makeSourceInfo(x,
-              "Simplified always-false condition"));
+          JsBlock block = new JsBlock(x.getSourceInfo());
           if (x.getInitExpr() != null) {
             block.getStatements().add(x.getInitExpr().makeStmt());
           }
@@ -331,7 +325,7 @@ public class JsStaticEval {
      * Simplify if statements.
      */
     @Override
-    public void endVisit(JsIf x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsIf x, JsContext ctx) {
       evalBooleanContext.remove(x.getIfExpr());
 
       JsExpression condExpr = x.getIfExpr();
@@ -353,35 +347,29 @@ public class JsStaticEval {
         ctx.replaceMe(condExpr.makeStmt());
       } else if (thenExpr != null && elseExpr != null) {
         // Convert "if (a()) {b()} else {c()}" => "a()?b():c()".
-        JsConditional cond = new JsConditional(makeSourceInfo(x,
-            "Replaced if statement with conditional"), x.getIfExpr(), thenExpr,
-            elseExpr);
+        JsConditional cond = new JsConditional(x.getSourceInfo(),
+            x.getIfExpr(), thenExpr, elseExpr);
         ctx.replaceMe(accept(cond.makeStmt()));
       } else if (thenIsEmpty && elseExpr != null) {
         // Convert "if (a()) {} else {b()}" => a()||b().
-        JsBinaryOperation op = new JsBinaryOperation(makeSourceInfo(x,
-            "Replaced if statement with ||"), JsBinaryOperator.OR,
-            x.getIfExpr(), elseExpr);
+        JsBinaryOperation op = new JsBinaryOperation(x.getSourceInfo(),
+            JsBinaryOperator.OR, x.getIfExpr(), elseExpr);
         ctx.replaceMe(accept(op.makeStmt()));
       } else if (thenIsEmpty && !elseIsEmpty) {
         // Convert "if (a()) {} else {stuff}" => "if (!a()) {stuff}".
         JsUnaryOperation negatedOperation = new JsPrefixOperation(
-            makeSourceInfo(x, "Simplified if with empty then statement"),
-            JsUnaryOperator.NOT, x.getIfExpr());
-        JsIf newIf = new JsIf(makeSourceInfo(x,
-            "Simplified if with empty then statement"), negatedOperation,
-            elseStmt, null);
+            x.getSourceInfo(), JsUnaryOperator.NOT, x.getIfExpr());
+        JsIf newIf = new JsIf(x.getSourceInfo(), negatedOperation, elseStmt,
+            null);
         ctx.replaceMe(accept(newIf));
       } else if (elseIsEmpty && thenExpr != null) {
         // Convert "if (a()) {b()}" => "a()&&b()".
-        JsBinaryOperation op = new JsBinaryOperation(makeSourceInfo(x,
-            "Replaced if statement with &&"), JsBinaryOperator.AND,
-            x.getIfExpr(), thenExpr);
+        JsBinaryOperation op = new JsBinaryOperation(x.getSourceInfo(),
+            JsBinaryOperator.AND, x.getIfExpr(), thenExpr);
         ctx.replaceMe(accept(op.makeStmt()));
       } else if (elseIsEmpty && elseStmt != null) {
         // Convert "if (a()) {b()} else {}" => "if (a()) {b()}".
-        JsIf newIf = new JsIf(makeSourceInfo(x, "Pruned empty else statement"),
-            x.getIfExpr(), thenStmt, null);
+        JsIf newIf = new JsIf(x.getSourceInfo(), x.getIfExpr(), thenStmt, null);
         ctx.replaceMe(accept(newIf));
       }
     }
@@ -390,7 +378,7 @@ public class JsStaticEval {
      * Change !!x to x in a boolean context.
      */
     @Override
-    public void endVisit(JsPrefixOperation x, JsContext<JsExpression> ctx) {
+    public void endVisit(JsPrefixOperation x, JsContext ctx) {
       if (x.getOperator() == JsUnaryOperator.NOT) {
         evalBooleanContext.remove(x.getArg());
       }
@@ -411,7 +399,7 @@ public class JsStaticEval {
      * Prune while (false) statements.
      */
     @Override
-    public void endVisit(JsWhile x, JsContext<JsStatement> ctx) {
+    public void endVisit(JsWhile x, JsContext ctx) {
       evalBooleanContext.remove(x.getCondition());
 
       JsExpression expr = x.getCondition();
@@ -420,8 +408,7 @@ public class JsStaticEval {
 
         // If false, replace with condition.
         if (cond.isBooleanFalse()) {
-          JsBlock block = new JsBlock(makeSourceInfo(x,
-              "Simplified always-false condition"));
+          JsBlock block = new JsBlock(x.getSourceInfo());
           block.getStatements().add(expr.makeStmt());
           JsStatement decls = ensureDeclarations(x.getBody());
           if (decls != null) {
@@ -433,31 +420,31 @@ public class JsStaticEval {
     }
 
     @Override
-    public boolean visit(JsConditional x, JsContext<JsExpression> ctx) {
+    public boolean visit(JsConditional x, JsContext ctx) {
       evalBooleanContext.add(x.getTestExpression());
       return true;
     }
 
     @Override
-    public boolean visit(JsDoWhile x, JsContext<JsStatement> ctx) {
+    public boolean visit(JsDoWhile x, JsContext ctx) {
       evalBooleanContext.add(x.getCondition());
       return true;
     }
 
     @Override
-    public boolean visit(JsFor x, JsContext<JsStatement> ctx) {
+    public boolean visit(JsFor x, JsContext ctx) {
       evalBooleanContext.add(x.getCondition());
       return true;
     }
 
     @Override
-    public boolean visit(JsIf x, JsContext<JsStatement> ctx) {
+    public boolean visit(JsIf x, JsContext ctx) {
       evalBooleanContext.add(x.getIfExpr());
       return true;
     }
 
     @Override
-    public boolean visit(JsPrefixOperation x, JsContext<JsExpression> ctx) {
+    public boolean visit(JsPrefixOperation x, JsContext ctx) {
       if (x.getOperator() == JsUnaryOperator.NOT) {
         evalBooleanContext.add(x.getArg());
       }
@@ -465,7 +452,7 @@ public class JsStaticEval {
     }
 
     @Override
-    public boolean visit(JsWhile x, JsContext<JsStatement> ctx) {
+    public boolean visit(JsWhile x, JsContext ctx) {
       evalBooleanContext.add(x.getCondition());
       return true;
     }
@@ -556,8 +543,7 @@ public class JsStaticEval {
       } else if (stmts.size() == 1) {
         return stmts.get(0);
       } else {
-        JsBlock jsBlock = new JsBlock(makeSourceInfo(stmt,
-            "Ensuring declarations"));
+        JsBlock jsBlock = new JsBlock(stmt.getSourceInfo());
         jsBlock.getStatements().addAll(stmts);
         return jsBlock;
       }
@@ -577,10 +563,6 @@ public class JsStaticEval {
       return num;
     }
 
-    private SourceInfo makeSourceInfo(HasSourceInfo x, String m) {
-      return x.getSourceInfo().makeChild(StaticEvalVisitor.class, m);
-    }
-
     private JsExpression simplifyEq(JsExpression original, JsExpression arg1,
         JsExpression arg2) {
       assert (original != null);
@@ -623,7 +605,7 @@ public class JsStaticEval {
         // "undefined" is not a JsValueLiteral, so the only way
         // the result can be true is if exp is itself a JsNullLiteral
         boolean result = exp instanceof JsNullLiteral;
-        return program.getBooleanLiteral(result);
+        return JsBooleanLiteral.get(result);
       }
 
       // no simplification made
@@ -640,7 +622,7 @@ public class JsStaticEval {
         // "undefined" is not a JsValueLiteral, so the only way
         // the result can be false is if exp is itself a JsNullLiteral
         boolean result = !(exp instanceof JsNullLiteral);
-        return program.getBooleanLiteral(result);
+        return JsBooleanLiteral.get(result);
       }
 
       // no simplification made
@@ -651,20 +633,20 @@ public class JsStaticEval {
      * Simplify a + b.
      */
     private void trySimplifyAdd(JsExpression original, JsExpression arg1,
-        JsExpression arg2, JsContext<JsExpression> ctx) {
+        JsExpression arg2, JsContext ctx) {
       if (arg1 instanceof JsValueLiteral && arg2 instanceof JsValueLiteral) {
+        SourceInfo info = original.getSourceInfo();
         // case: number + number
         if (arg1 instanceof JsNumberLiteral && arg2 instanceof JsNumberLiteral) {
           double value = ((JsNumberLiteral) arg1).getValue()
               + ((JsNumberLiteral) arg2).getValue();
-          ctx.replaceMe(program.getNumberLiteral(value));
+          ctx.replaceMe(new JsNumberLiteral(info, value));
         } else {
           // cases: number + string or string + number
           StringBuilder result = new StringBuilder();
           if (appendLiteral(result, (JsValueLiteral) arg1)
               && appendLiteral(result, (JsValueLiteral) arg2)) {
-            ctx.replaceMe(program.getStringLiteral(original.getSourceInfo(),
-                result.toString()));
+            ctx.replaceMe(new JsStringLiteral(info, result.toString()));
           }
         }
       }
@@ -676,7 +658,7 @@ public class JsStaticEval {
      * expressions as left-normal as possible.
      */
     private boolean trySimplifyAssociativeExpression(JsBinaryOperation x,
-        JsContext<JsExpression> ctx) {
+        JsContext ctx) {
       boolean toReturn = false;
       JsBinaryOperator op = x.getOperator();
       JsExpression arg1 = x.getArg1();
@@ -770,7 +752,7 @@ public class JsStaticEval {
     }
 
     private void trySimplifyEq(JsExpression original, JsExpression arg1,
-        JsExpression arg2, JsContext<JsExpression> ctx) {
+        JsExpression arg2, JsContext ctx) {
       JsExpression updated = simplifyEq(original, arg1, arg2);
       if (updated != original) {
         ctx.replaceMe(updated);
@@ -778,20 +760,18 @@ public class JsStaticEval {
     }
 
     private void trySimplifyNe(JsExpression original, JsExpression arg1,
-        JsExpression arg2, JsContext<JsExpression> ctx) {
+        JsExpression arg2, JsContext ctx) {
       JsExpression updated = simplifyNe(original, arg1, arg2);
       if (updated != original) {
         ctx.replaceMe(updated);
       }
     }
 
-    private boolean tryStaticEvalIf(JsIf x, CanBooleanEval cond,
-        JsContext<JsStatement> ctx) {
+    private boolean tryStaticEvalIf(JsIf x, CanBooleanEval cond, JsContext ctx) {
       JsStatement thenStmt = x.getThenStmt();
       JsStatement elseStmt = x.getElseStmt();
       if (cond.isBooleanTrue()) {
-        JsBlock block = new JsBlock(makeSourceInfo(x,
-            "Simplified always-true condition"));
+        JsBlock block = new JsBlock(x.getSourceInfo());
         block.getStatements().add(x.getIfExpr().makeStmt());
         if (thenStmt != null) {
           block.getStatements().add(thenStmt);
@@ -803,8 +783,7 @@ public class JsStaticEval {
         ctx.replaceMe(accept(block));
         return true;
       } else if (cond.isBooleanFalse()) {
-        JsBlock block = new JsBlock(makeSourceInfo(x,
-            "Simplified always-false condition"));
+        JsBlock block = new JsBlock(x.getSourceInfo());
         block.getStatements().add(x.getIfExpr().makeStmt());
         if (elseStmt != null) {
           block.getStatements().add(elseStmt);
@@ -831,18 +810,17 @@ public class JsStaticEval {
       JsBinaryOperator.BIT_OR, JsBinaryOperator.BIT_XOR,
       JsBinaryOperator.COMMA, JsBinaryOperator.MUL, JsBinaryOperator.OR);
 
-  @SuppressWarnings("unchecked")
   public static <T extends JsVisitable> T exec(JsProgram program, T node) {
-    Event optimizeJsEvent = SpeedTracerLogger.start(CompilerEventType.OPTIMIZE_JS, 
-        "optimizer", NAME);
+    Event optimizeJsEvent = SpeedTracerLogger.start(
+        CompilerEventType.OPTIMIZE_JS, "optimizer", NAME);
     T result = new JsStaticEval(program).execImpl(node);
     optimizeJsEvent.end();
     return result;
   }
 
   public static OptimizerStats exec(JsProgram program) {
-    Event optimizeJsEvent = SpeedTracerLogger.start(CompilerEventType.OPTIMIZE_JS, 
-        "optimizer", NAME);
+    Event optimizeJsEvent = SpeedTracerLogger.start(
+        CompilerEventType.OPTIMIZE_JS, "optimizer", NAME);
     OptimizerStats stats = new JsStaticEval(program).execImpl();
     optimizeJsEvent.end("didChange", "" + stats.didChange());
     return stats;
@@ -902,7 +880,7 @@ public class JsStaticEval {
    * </pre>
    */
   protected static void shortCircuitAnd(JsExpression arg1, JsExpression arg2,
-      JsContext<JsExpression> ctx) {
+      JsContext ctx) {
     if (arg1 instanceof CanBooleanEval) {
       CanBooleanEval eval1 = (CanBooleanEval) arg1;
       if (eval1.isBooleanTrue() && !arg1.hasSideEffects()) {
@@ -922,7 +900,7 @@ public class JsStaticEval {
    * </pre>
    */
   protected static void shortCircuitOr(JsExpression arg1, JsExpression arg2,
-      JsContext<JsExpression> ctx) {
+      JsContext ctx) {
     if (arg1 instanceof CanBooleanEval) {
       CanBooleanEval eval1 = (CanBooleanEval) arg1;
       if (eval1.isBooleanTrue()) {
@@ -934,7 +912,7 @@ public class JsStaticEval {
   }
 
   protected static void trySimplifyComma(JsExpression arg1, JsExpression arg2,
-      JsContext<JsExpression> ctx) {
+      JsContext ctx) {
     if (!arg1.hasSideEffects()) {
       ctx.replaceMe(arg2);
     }
@@ -946,7 +924,6 @@ public class JsStaticEval {
     this.program = program;
   }
 
-  @SuppressWarnings("unchecked")
   public <T extends JsVisitable> T execImpl(T node) {
     return new StaticEvalVisitor().accept(node);
   }
