@@ -1,12 +1,12 @@
 /*
  * Copyright 2008 Google Inc.
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License"); you may not
  * use this file except in compliance with the License. You may obtain a copy of
  * the License at
- * 
+ *
  * http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
  * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
@@ -29,6 +29,14 @@ public class Color extends Properties {
     return JavaScriptObject.createObject().cast();
   }
 
+  public static Color create(String fill, String stroke, int strokeSize) {
+    Color result = create();
+    result.setFill(fill);
+    result.setStroke(stroke);
+    result.setStrokeSize(strokeSize);
+    return result;
+  }
+
   protected Color() {
   }
 
