@@ -1,6 +1,6 @@
 /*
  * Isomorphic SmartClient
- * Version 8.0 (2010-05-18)
+ * Version SC_SNAPSHOT-2010-12-31 (2010-12-31)
  * Copyright(c) 1998 and beyond Isomorphic Software, Inc. All rights reserved.
  * "SmartClient" is a trademark of Isomorphic Software, Inc.
  *
@@ -588,7 +588,7 @@ isc.Dialog.Prompt = {
     bodyStyle:"promptBody", // no border-top, since there is no header
                             // TODO autogenerate border in Window based on header visibility
     
-    message:"Loading...",
+    message:"Loading...&nbsp;${loadingImage}",
 
     blurbDefaults : {width:390, align:isc.Canvas.CENTER, valign:isc.Canvas.CENTER, canSelectText: true},
     
@@ -608,9 +608,11 @@ isc.Dialog.Prompt = {
     
 		// first add the properties specified
 		this.setProperties(properties);
-    
-        this.message = newMessage;
-
+        this.message = newMessage.evalDynamicString(this, {
+            loadingImage: this.imgHTML(isc.Canvas.loadingImageSrc, 
+                                       isc.Canvas.loadingImageSize, 
+                                       isc.Canvas.loadingImageSize)
+            });
         // Note: we lazily create children on draw, so verify that the items have been
         // initialized before manipulating the label
         if (!this._isInitialized) this.createChildren();
@@ -619,7 +621,7 @@ isc.Dialog.Prompt = {
         this.addAutoChild("blurb", null, isc.Label, this.body);
         // Support custom styling of the blurb        
         if (this.messageStyle != null) this.blurb.setBaseStyle(this.messageStyle); 
-        this.blurb.setContents(this.message);
+        this.blurb.setContents(this.message == null ? "&nbsp;" : this.message);
 		
 		this.show();
 	},
@@ -648,6 +650,7 @@ isc.Dialog.Prompt = {
 //  advise calling this method, then using +link{Class.delayCall()} or +link{Timer.setTimeout}
 //  to kick off the slow logic in a separate thread. This ensures that the prompt is showing
 //  before the lengthy execution begins.
+//  <p/>Use <code>"\${loadingImage}"</code> to include +link{Canvas.loadingImageSrc,a loading image}.
 //  
 //
 //	@param	message			(string)	message to display
@@ -794,7 +797,7 @@ isc.Dialog.Warn = {
 
         // Update the label in the body        
         if (this.messageStyle != null) this.blurb.setBaseStyle(this.messageStyle);
-        this.blurb.setContents(this.message);
+        this.blurb.setContents(this.message == null ? "&nbsp;" : this.message);
 
         if (this.icon) {
             this.iconImg.setSrc(this.getImgURL(this.icon));
@@ -889,6 +892,20 @@ isc.addGlobal("showMessage", function (message, messageType, callback, propertie
     isc.Dialog.Warn.showMessage(message, properties);
 });
 
+//> @classMethod isc.getLastDialog()
+// Returns the last-shown isc.say/ask/warn/confirm dialog.  Do not document externally.
+//<
+isc.addGlobal("getLastDialog", function () {
+    return isc.Dialog.Warn;
+});
+
+//> @classMethod isc.dismissLastDialog()
+// Dismisses the last-shown isc.say/ask/warn/confirm dialog.  Do not document externally.
+//<
+isc.addGlobal("dismissLastDialog", function () {
+    if (isc.Dialog.Warn) isc.Dialog.Warn.hide();
+});
+
 // shared with askForValue()
 isc._applyDialogHandlers = function (properties) {
     
@@ -1274,14 +1291,16 @@ isc.LoginDialog.addProperties({
     //<
 
     //> @attr loginDialog.dismissable (Boolean : false : [IR])
-    // If true, allow the user to close the LoginDialog by pressing Escape or by pressing
-    // a Close button on the upper right corner (only visible when true).
-    // <p>If the Dialog is dismissed,
-    // +link{LoginDialog.loginFunc} is called with null arguments.
-    // <p>Note that this attribute overrides the dismissOnEscape and showCloseButton
-    // attributes.
+    // Whether the user should be able to dismiss the login dialog without entering
+    // credentials.  Set to true if logging in is optional.  When set, a close button will be
+    // present, and hitting escape will also dismiss the dialog.
+    // <p>
+    // If the Dialog is dismissed, +link{LoginDialog.loginFunc} is called with null arguments.
+    // <p>
+    // Note that this attribute overrides the dismissOnEscape and showCloseButton attributes.
     // @visibility external
     //<
+    dismissable: false,
     
     //> @attr   loginDialog.dismissOnEscape  (boolean : null : [IRW])
     // Do not set LoginDialog.dismissOnEscape; it is controlled by the 
@@ -1297,8 +1316,6 @@ isc.LoginDialog.addProperties({
     // @visibility external
     //<
     
-    dismissable: false,
-    
     //> @attr loginDialog.allowBlankPassword (Boolean : false : IR)
     // If true, the login form will allow blank passwords to be submitted. Otherwise
     // the form fails to be validated until the user enters at least one character into
@@ -1516,7 +1533,12 @@ isc.LoginDialog.addProperties({
     },
     formDefaultFields: [
         { name: "loginFailureItem", type:"blurb", colSpan: 2, visible:false },
-        { name: "usernameItem", required:true, 
+        { name: "usernameItem", required:true,
+            // Diable spell checking etc where supported
+            browserSpellCheck:false,
+            browserAutoCorrect:false,
+            browserAutoCapitalize:false,
+            
             keyPress : function (item, form, keyName) {
                 if (keyName == "Enter") {
                     form.focusInItem("passwordItem");
