@@ -1,6 +1,6 @@
 /*
  * Isomorphic SmartClient
- * Version 8.0 (2010-05-18)
+ * Version SC_SNAPSHOT-2010-12-31 (2010-12-31)
  * Copyright(c) 1998 and beyond Isomorphic Software, Inc. All rights reserved.
  * "SmartClient" is a trademark of Isomorphic Software, Inc.
  *
@@ -153,7 +153,7 @@ isc.defineClass("PortalColumnHeader", "HLayout").addProperties({
 
         this.addMember(isc.LayoutSpacer.create());
 
-        this.menu = isc.Menu.create({
+        this.menu = this.getMenuConstructor().create({
             width: 150,
             portalColumn: this.portalColumn,
             data:[
@@ -314,7 +314,7 @@ isc.defineClass("PortalColumn", "Layout").addProperties({
     
     rowConstructor:"PortalRow",
     addPortlet : function (portlet, position) {
-        
+
         // offset position to be position within rows
         if (this.showColumnHeader) position += 1;
         
@@ -342,6 +342,21 @@ isc.defineClass("PortalColumn", "Layout").addProperties({
         
         portalRow.addMember(portlet);
     },
+
+    addPortletToExistingRow : function (portlet, rowNum, rowOffset) {
+
+        // offset position to be position within rows
+        if (this.showColumnHeader) rowNum += 1;
+
+        var rows = this.getMembers();
+
+        if (rows == null || rows.length <= rowNum) {
+            this.addPortlet(portlet);
+        } else {
+            var portalRow = this.getMember(rowNum);
+            portalRow.addMember(portlet, rowOffset);
+        }
+    },
     
     getDropPosition : function () {
         var dropPosition = this.Super("getDropPosition", arguments);
@@ -503,6 +518,10 @@ isc.defineClass("PortalLayout", "Layout").addProperties({
         if (column != null) column.addPortlet(portlet, rowNum);
     },
     
+    getColumn : function (colNum) {
+        return this.getMember(colNum);
+    },
+    
     //>@method portalLayout.removePortlet()
     // Removes a +link{Portlet} which is currently rendered in this PortalLayout.
     // Portlet will not be destroyed by default - if this is desired the calling code should
