@@ -1,6 +1,6 @@
 /*
  * Isomorphic SmartClient
- * Version 8.0 (2010-05-18)
+ * Version SC_SNAPSHOT-2010-12-31 (2010-12-31)
  * Copyright(c) 1998 and beyond Isomorphic Software, Inc. All rights reserved.
  * "SmartClient" is a trademark of Isomorphic Software, Inc.
  *
@@ -332,7 +332,28 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
         
         return this.Super("handleKeyDown", arguments);
     },
+    
+    // Expose getEnteredValue() - commonly useful for pickLists
+    //> @method comboBoxItem.getEnteredValue()
+    // @include textItem.getEnteredValue()
+    // @visibility external
+    //<
 
+    //> @attr comboBoxItem.addUnknownValues (boolean : true : IRW)
+    // If set to false, if a value has been entered that does not match an entry in the
+    // +link{valueMap} or a value loaded from the +link{optionDataSource}, it will be discarded if the
+    // user leaves the field.
+    // <P>
+    // When this mode is enabled, +link{getValue()} will return null unless the value typed in by the
+    // user has valid matches.  Use +link{getEnteredValue()} to get the raw value typed in by the
+    // user.
+    // <P>
+    // Note that this flag effectively enables +link{completeOnTab}, since leaving the field in
+    // general (whether by tab or another means) will attempt completion, and discard the value if no
+    // valid completion exists.  As with <code>completeOnTab</code>, if the typed in value has more
+    // than one match, the first match will be used.
+    // @visibility external
+    //<
     addUnknownValues:true,
 
     // when addUnknownValues is false, on field exit, drop any value that doesn't have a valid
@@ -360,19 +381,26 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
 
     // when addUnknownValues is false, don't allow values that have no valid completions to
     // become this item's value, trigger change()/changed() or be saved to the form
+    
     _updateValue : function (value, forceSave) {
-        if (this.addUnknownValues || forceSave) return this.Super("_updateValue", arguments);
-
+        
         // when a value is selected, the element value is changed to show the display value.
         // We will receive change notifications for this (eg onblur) but this should not be
         // considered a change to the underlying value of the FormItem
-        var elementValue = this.getElementValue();
-        if (value == elementValue && this.mapValueToDisplay(this.getValue()) == elementValue) {
-            return;
+        var elementValue = this.getElementValue(),
+            unchanged = (value == elementValue && this.mapValueToDisplay(this.getValue()) == elementValue);
+            
+        // _displayValueModified - this implies the user changed the value
+        // by typing into the text box rather than picking a value from the
+        // pickList.
+        this._displayValueModified = (!unchanged && !this._valuePicked);
+        
+        if (this.addUnknownValues || forceSave) {
+            return this.Super("_updateValue", arguments);
         }
+        if (unchanged) return;
 
         var displayValue = this.mapValueToDisplay(value);
-        //this.logWarn("_updateValue: value: " + value + " has display value: " + displayValue);
         if (displayValue == value) {
             // if the user-entered value doesn't match anything in the current pickList, the
             // official value is now considered null
@@ -469,6 +497,27 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
         this.pickList.clearLastHilite();
         this.pickList.scrollRecordIntoView(0);
     },
+    
+    // Override getSelectedRecord to look at the pickList if present
+    getSelectedRecord : function () {
+          
+        var record = this.Super("getSelectedRecord", arguments);
+     
+        
+        // If we didn't get selected record via 'fetchMissingValues',
+        // use the pickList to try to find the selectedRecord in the pickList data
+        if (record == null && this._value != null && this.getOptionDataSource()) {
+            if (this.pickList == null || this.pickList.destroyed) this.makePickList(false);
+    
+            if (this.pickList && this.pickList.data) {
+                record = this.pickList.data.find(this.getValueFieldName(), this._value);
+                if (record != null) this._storeSelectedRecord(record);
+            }
+        }
+        return record;
+    },        
+        
+        
 
     // ------------------------
     // Filtering data
@@ -521,10 +570,10 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
     //> @attr ComboBoxItem.filterLocally
     // @include PickList.filterLocally
     // @visibility external
-    //<    
+    //<
 
     //> @method ComboBoxItem.getSelectedRecord()
-    // @include PickList.getSelectedRecord()
+    // @include FormItem.getSelectedRecord()
     // @visibility external
     //<
     
@@ -643,7 +692,7 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
     // @include PickList.filterClientPickListData()
     //<
 
-    //> @attr comboBoxItem.textMatchStyle (String : "startsWith" : IR)
+    //> @attr comboBoxItem.textMatchStyle (TextMatchStyle : "startsWith" : IR)
     // @include PickList.textMatchStyle
     //<
 
@@ -705,8 +754,8 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
         delete this._showOnFilter;
         delete this._showOnDelayedFilter;
 
-        // bail if the pickList isn't showing 
-        if (!pickList || !pickList.isVisible()) return;
+        // bail if the pickList isn't showing or has been passed to another form item already
+        if (!pickList || !pickList.isVisible() || pickList.formItem != this) return;
 
         // hide the pickList
         pickList.hide();
@@ -775,6 +824,10 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
     // a value that's not currently loaded in the pickList dataSet, we show the display value
     // rather than the underlying data value.
     setValue : function (newValue, a,b,c,d) {
+        
+        // if 'setValue' is called - drop the "displayValueModified" flag
+        this._displayValueModified = false;
+        
         this._programmaticSetValue = true;
         this.invokeSuper(isc.ComboBoxItem, "setValue", newValue, a,b,c,d);
         delete this._programmaticSetValue;
@@ -803,9 +856,19 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
 
     // Override getCriteriaFieldName - if we have a displayField and no currently selected 
     // record, return the displayField name; otherwise, just return the item name
+    // The concept here is that the user can type a partial displayField value and we'd want
+    // to perform searches etc based on that field.
+    // However - do check the _displayValueModified flag too -- if this is false then
+    // a programmatic setValue() may have been called -- in this case we'll only
+    // have a selectedRecord if one's been loaded from the dataSource (may be pending that fetch)
+    // but we still want to consider the value a "data" value, not a displayField value.
     getCriteriaFieldName : function () {
-        if (this.displayField && !this.getSelectedRecord()) return this.displayField;
-        return this.getFieldName();
+        
+        var showingDisplayValue = this._displayValueModified && !this.getSelectedRecord();
+        if (this.displayField && showingDisplayValue) return this.displayField;
+        // Note: DO NOT CALL Super.getCriteriaFieldName() - we subclass TextItem which returns
+        // displayField if set
+        return this.criteriaField || this.getFieldName();
     },
     
     // Override checkForDisplayFieldValue()
@@ -855,7 +918,9 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
         
         if (this._pickedReverseValueMap) {
             for (var i in this._pickedReverseValueMap) {
-                if (i == value) return this._pickedReverseValueMap[i];
+                if (i == value) {
+                    return this._pickedReverseValueMap[i];
+                }
             }
             delete this._pickedReverseValueMap;
         }
@@ -864,7 +929,6 @@ isc.defineClass("ComboBoxItem", "TextItem", "PickList").addMethods({
             displayValue = this._translateValueFieldValue(value, true);
             if (displayValue != null) value = displayValue;
         }
-            
         return this.invokeSuper(isc.ComboBoxItem, "mapDisplayToValue", value, a,b,c);
     }
 });
