@@ -1,6 +1,6 @@
 /*
  * Isomorphic SmartClient
- * Version 8.0 (2010-05-18)
+ * Version SC_SNAPSHOT-2010-12-31 (2010-12-31)
  * Copyright(c) 1998 and beyond Isomorphic Software, Inc. All rights reserved.
  * "SmartClient" is a trademark of Isomorphic Software, Inc.
  *
@@ -200,16 +200,32 @@ getItems : function () {
 // override getTitleHTML() to avoid writing a <label> tag around the title, and setting an 
 // accessKey property on that label.  This is appropriate as we are setting up the accessKey
 // directly on the first sub-element in the group (implemented in setItems)
-getTitleHTML : function () {
-
+getTitleHTML : function(){
+    var elementID, focusableItem;
     var title = this.getTitle();
     
+    if (!this.getCanFocus()) {
+        return title;
+    }
+    
     if (this.accessKey != null) {
         title = isc.Canvas.hiliteCharacter(title, this.accessKey);
     }
-            
-    return title;
+    
+    for (var i = 0; i < this.items.length; i++) {
+        if (this.items[i].getCanFocus() && this.items[i].hasDataElement()) {
+            focusableItem = this.items[i];
+            break;
+        }
+    }
+    
+    if (!focusableItem) {
+        return title;
+    }
+    
+    return isc.SB.concat("<LABEL FOR=", focusableItem.getDataElementId(), ">", title, "</LABEL>");
 },
+  
 
 // Override '_setElementTabIndex()', as the superclass implementation forces a form.redraw()
 // for any focusable items without elements, and this may not be necessary.
@@ -362,7 +378,6 @@ makeNamedItem : function (itemName, extraProperties) {
 //		@param	values	(string)	Value of the element (unused).
 //		@return			(string)	HTML to draw this item.
 //<
-_$suppressDoublingCSS:isc.Canvas._$noStyleDoublingCSS,
 getInnerHTML : function (values, includeHint, includeErrors, returnArray) {
     
 	if (!values) values = {};
@@ -527,7 +542,7 @@ getInnerHTML : function (values, includeHint, includeErrors, returnArray) {
                 // can leave height, extrastuff unspecified
                 null, null, 
                 // Avoid double border/margin/padding
-                this._$suppressDoublingCSS,
+                isc.Canvas._$noStyleDoublingCSS,
                 // Don't pass in itemID or formID - don't write out a click handler on this cell
                 null, null, null,
                 // Suppress wrapping of icons (if we have more than one in the cell)
