@@ -1,6 +1,6 @@
 /*
  * Isomorphic SmartClient
- * Version 8.0 (2010-05-18)
+ * Version SC_SNAPSHOT-2010-12-31 (2010-12-31)
  * Copyright(c) 1998 and beyond Isomorphic Software, Inc. All rights reserved.
  * "SmartClient" is a trademark of Isomorphic Software, Inc.
  *
@@ -26,13 +26,14 @@ if (isc.ListGrid) {
 isc.ClassFactory.defineInterface("PickList");
 
 //>	@class PickListMenu
-// Subclass of ScrollingMenu used by FormItems which implement +link{interface:PickList} to display
-// a list of selectable options.<br>
-// Note that PickListMenu ultimately inherits from +link{class:ListGrid}.
+// +link{class:ListGrid} subclass used by FormItems which implement
+// +link{interface:PickList} to display
+// a list of selectable options.
 //  @treeLocation Client Reference/Forms/Form Items
 // @visibility external
 //<
 
+
 isc.ClassFactory.defineClass("PickListMenu", "ScrollingMenu");
 
 isc.PickListMenu.addClassProperties({
@@ -70,8 +71,23 @@ isc.PickListMenu.addProperties({
     // Since we don't support drag resize of fields, turn canFreezeFields off by default
     canFreezeFields:false,
     
+    //> @attr pickListMenu.styleName (CSSStyleName : "pickListMenu" : IRW)
+    // @include listGrid.styleName
+    // @visibility external
+    //<
     styleName:"pickListMenu",
-    bodyStyleName:"pickListMenuBody"
+    //> @attr pickListMenu.bodyStyleName (CSSStyleName : "pickListMenuBody" : IRW)
+    // @include listGrid.bodyStyleName
+    // @visibility external
+    //<
+    bodyStyleName:"pickListMenuBody",
+    
+    
+    //> @attr pickListMenu.normalCellHeight (number : 16 : IRWA)
+    // @include listGrid.normalCellHeight
+    //<
+    normalCellHeight:16
+    
     
 });
 
@@ -120,40 +136,57 @@ isc.PickListMenu.addMethods({
         if (record != null) this.itemClick(record);
         
     },
+    headerClick : function (fieldNum, header) {
+        var rv = this.Super("headerClick", arguments);
+        var field = this.getField(fieldNum);
+        // check if the checkbox column header was clicked
+        if (this.isCheckboxField(field) && this.allowMultiSelect) {
+            this.multiSelectChanged();
+        }
+        return rv;
+    },
+    
+    multiSelectChanged : function () {
+        var formItem = this.formItem,
+            fieldName = formItem.getValueFieldName(),
+            sel = this.getSelection(),
+            values = [];
+            
+         for (var i = 0; i < sel.length; i++) {
+             var currSel = sel[i];
+             values.add(currSel[fieldName]);    
+         }       
+         formItem.pickValue(values);
+    },
     
     // 'pick' the selected value on click.  Matches Windows ComboBox behavior
     itemClick : function (record) {
-        var formItem = this.formItem,
-            fieldName = formItem.getValueFieldName();            
-
         if (this.allowMultiSelect) {
-             var sel = this.getSelection();
-             var values = [];
-             for (var i = 0; i < sel.length; i++) {
-                 var currSel = sel[i];
-                 values.add(currSel[fieldName]);    
-             }       
-             formItem.pickValue(values);
-         } else {
-         	 var value = record[fieldName];
-             formItem.pickValue(value);
-         }
+            this.multiSelectChanged();
+        } else {
+            var formItem = this.formItem,
+                fieldName = formItem.getValueFieldName();       
+         	var value = record[fieldName];
+            formItem.pickValue(value);
+        }
     },
 
-    hide : function (a,b,c,d) {         
+    hide : function (a,b,c,d) {
+        var isVisible = this.isVisible() && this.isDrawn();
+        
         this.invokeSuper(isc.PickListMenu, "hide", a,b,c,d);        
         // If we're being hidden as part of a formItem.destroy(), this.formItem will have been 
         // cleared out.
         if (!this.formItem) return;
         
         // put focus back in the item if this was a modal pickList 	 
-        if (this.showModal) this.formItem.focusInItem();
+        if (isVisible && this.showModal) this.formItem.focusInItem();
                  
         // Clear out the showingPickList flag
         this.formItem._showingPickList = null;                 
         
         // fire a notification for observing / overriding the pick list being hidden
-        this.formItem._pickListHidden();
+        if (isVisible) this.formItem._pickListHidden();
         
         
         delete this.formItem._showOnFilter;
@@ -366,10 +399,25 @@ isc.PickList.addInterfaceProperties({
     //> @attr    PickList.pickListBaseStyle (string : "pickListCell" : IR)
     // Base Style for pickList cells.  As with ListGrid Cells, will have 'over', 'selected'
     // and 'disabled' appended on changes of state for the cells.
+    // <p>
+    // Note: if +link{pickListTallBaseStyle} is specified, this property will be used as the
+    // +link{listGrid.normalBaseStyle,normalBaseStyle} and that property will be applied
+    // to cells that do not match the default cell height, or if +link{listGrid.fastCellUpdates}
+    // is true for the pickList in Internet Explorer.
+    //
     // @visibility external
     //<
     pickListBaseStyle : "pickListCell",
-
+    
+    //> @attr    PickList.pickListTallBaseStyle (string : null : IR)
+    // Optional +link{listGrid.tallBaseStyle,tallBaseStyle} for pickListCells. If unset
+    // +link{pickListBaseStyle} will be applied to all cells.
+    //
+    // @visibility external
+    //<
+    //pickListTallBaseStyle : null,
+    
+    
     //> @attr    PickList.pickListHiliteColor  (string : null : IR)
     // If specified this color will be applied to hilited rows in the pickList, when the
     // user rolls over them.  This color will be applied on top of the "over" CSS Style 
@@ -863,7 +911,11 @@ isc.PickList.addInterfaceMethods({
     
     getPickListCellHeight : function () {
         var cellHeight = this.pickListCellHeight;
-        if (this.valueicons != null || this.getValueIcon != null) {
+        // If a developer specifies pickListProperties.cellHeight, respect that too
+        if (this.pickListProperties && this.pickListProperties.cellHeight != null) {
+            cellHeight = this.pickListProperties.cellHeight;
+        }
+        if (this.valueIcons != null || this.getValueIcon != null) {
             var valueIconHeight = this.getValueIconHeight();
             if (valueIconHeight > cellHeight) cellHeight = valueIconHeight;
         }
@@ -879,7 +931,7 @@ isc.PickList.addInterfaceMethods({
         var pickList = this.pickList;
        
         
-        var cellHeight = this.pickListCellHeight;
+        var cellHeight = this.getPickListCellHeight();
         pickList.setCellHeight(cellHeight);     
         
         // These methods both no-op if the pickList is already applied to this item
@@ -908,12 +960,11 @@ isc.PickList.addInterfaceMethods({
         isc.addProperties(pickListProperties, {
             // Ensure there's a pointer back to the form item
             formItem:this,
-            // if the cellHeight for the picklist is greater than the normal cell
-            // height, use the default listgrid css class instead ('cell')
-            // The pickListBaseStyle in some skins uses media specifically designed
-            // for the default height, which looks bad for larger heights.
-            baseStyle: (this.pickList.cellHeight > this.pickList.normalCellHeight) ? 
-                "cell" : this.pickListBaseStyle,
+            
+            normalBaseStyle:this.pickListBaseStyle,
+            // If a tall style is defined use it, otherwise assume the normal version
+            // can stretch
+            tallBaseStyle:(this.pickListTallBaseStyle || this.pickListBaseStyle),
             
             hiliteColor:this.pickListHiliteColor,
             hiliteTextColor:this.pickListHiliteTextColor,
@@ -931,7 +982,7 @@ isc.PickList.addInterfaceMethods({
             dataArrived : function (startRow, endRow) {
                 if (isc._traceMarkers) arguments.__this = this;
                 this.Super("dataArrived", arguments);
-                if (this.formItem) this.formItem.handleDataArrived(startRow,endRow,this);
+                if (this.formItem) this.formItem.handleDataArrived(startRow,endRow,this.data);
             }
         });
         if (this.multiple && this.multipleAppearance == "picklist" 
@@ -1464,8 +1515,8 @@ isc.PickList.addInterfaceMethods({
             } else {
                 record = data.find(valueField, currVal);            
             }
-    
-            if (record) {
+
+            if (record && record != Array.LOADING) {
                 
                 if (this.pickList.allowMultiSelect) this.pickList.selectRecord(record);
                 else this.pickList.selection.selectSingle(record);
@@ -1500,7 +1551,8 @@ isc.PickList.addInterfaceMethods({
             } else {
                 list.setHeight(this.getPickListHeight());
                 var position = this.getPickListPosition();
-                list.placeNear(position[0], position[1]);
+                if (this.allowPickListToClip) list.setRect([position[0], position[1]]);
+                else list.placeNear(position[0], position[1]);
             }
 		} else {
             // if we set the flag to show the list after the filter, show it now!
@@ -1629,6 +1681,9 @@ isc.PickList.addInterfaceMethods({
         if (this.defaultToFirstOption && this.getValue() == null && startRow == 0) {
             this.setToDefaultValue();
         }
+         // helper - if we're currently showing a data value and we just loaded the associated
+        // display value, display it.
+        this._updateDisplayValueForNewData();
 
         if (this.dataArrived) this.dataArrived(startRow,endRow,data);
     },
@@ -1645,10 +1700,8 @@ isc.PickList.addInterfaceMethods({
     
     // -- Client side filtering --
     
-    //> @attr PickList.textMatchStyle (String : "startsWith" : IR)
-    // When applying filter criteria to pickList data, what type of matching to use.  Legal
-    // values are "substring" (value contains user input) or "startsWith" (value starts with
-    // user input).  Both matches are case insensitive.
+    //> @attr PickList.textMatchStyle (TextMatchStyle : "startsWith" : IR)
+    // When applying filter criteria to pickList data, what type of matching to use.
     // <P>
     // For a databound pickList (+link{optionDataSource} set), <code>textMatchStyle</code> is
     // sent to the server as +link{dsRequest.textMatchStyle}.
@@ -1707,42 +1760,50 @@ isc.PickList.addInterfaceMethods({
         
         var validCriterion = false;
         for (var fieldName in criteria) {
-            var searchString = criteria[fieldName];
-            if (!searchString || isc.isA.emptyString(searchString)) continue;
+            var fieldCriterion = criteria[fieldName];
+            if (!fieldCriterion || isc.isA.emptyString(fieldCriterion)) continue;
 
             validCriterion = true;
             
-            // Do a raw conversion to strings if passed non string values
+            // Handle being passed an array
             
-            if (!isc.isA.String(searchString)) searchString += isc.emptyString;
+            if (!isc.isAn.Array(fieldCriterion)) fieldCriterion = [fieldCriterion];
             
-            searchString = searchString.toLowerCase();
-            
-            var dataLength = data.getLength(),
-                valueFieldName = this.getValueFieldName();
-            for (var i = 0; i < dataLength; i++) {
-                var possibleMatch = data[i][fieldName];
-                // for the valueField, run the record values through mapValueToDisplay() so
-                // they match what the users sees.  XXX running through the valueMap is
-                // appropriate here, but a formatter that returns HTML may screw this up
-                if (this.filterDisplayValue && fieldName == valueFieldName) {
-                    possibleMatch = this.mapValueToDisplay(possibleMatch);
-                }
+            for (var stringIndex = 0; stringIndex < fieldCriterion.length; stringIndex++) {
+                var searchString = fieldCriterion[stringIndex];
                 
-                // For now we'll do a stringwise comparison regardless of the data type of
-                // the possible match
-                if (!isc.isA.String(possibleMatch)) possibleMatch += "";
-            
-                possibleMatch = possibleMatch.toLowerCase();
-                // Remove any mismatches from the list of options
-                if ((this.textMatchStyle == this._$substring &&
-                     !possibleMatch.contains(searchString)) ||
-                    (this.textMatchStyle != this._$substring && // assume startsWith
-                     !isc.startsWith(possibleMatch, searchString))) 
-                {
-                    if (this.showAllOptions) nonMatches.add(data[i]);
-                } else {
-                    matches.add(data[i]);
+                // Do a raw conversion to strings if passed non string values
+                
+                if (!isc.isA.String(searchString)) searchString += isc.emptyString;
+                
+                searchString = searchString.toLowerCase();
+                
+                var dataLength = data.getLength(),
+                    valueFieldName = this.getValueFieldName();
+                for (var i = 0; i < dataLength; i++) {
+                    var possibleMatch = data[i][fieldName];
+                    // for the valueField, run the record values through mapValueToDisplay() so
+                    // they match what the users sees.  XXX running through the valueMap is
+                    // appropriate here, but a formatter that returns HTML may screw this up
+                    if (this.filterDisplayValue && fieldName == valueFieldName) {
+                        possibleMatch = this.mapValueToDisplay(possibleMatch);
+                    }
+                    
+                    // For now we'll do a stringwise comparison regardless of the data type of
+                    // the possible match
+                    if (!isc.isA.String(possibleMatch)) possibleMatch += "";
+                
+                    possibleMatch = possibleMatch.toLowerCase();
+                    // Remove any mismatches from the list of options
+                    if ((this.textMatchStyle == this._$substring &&
+                         !possibleMatch.contains(searchString)) ||
+                        (this.textMatchStyle != this._$substring && // assume startsWith
+                         !isc.startsWith(possibleMatch, searchString))) 
+                    {
+                        if (this.showAllOptions) nonMatches.add(data[i]);
+                    } else {
+                        matches.add(data[i]);
+                    }
                 }
             }
         }
@@ -1900,29 +1961,10 @@ isc.PickList.addInterfaceMethods({
         // pickList onscreen without obscuring the select item body by placing above the item
         // if necessary... for other items using the pickList interface, this placeNear will
         // just position the pickList as close to the specified position as possible.
-        pickList.placeNear(left, top);
+        if (this.allowPickListToClip) pickList.setRect([left, top]);
+        else pickList.placeNear(left, top);
     },
 
-    //> @method PickList.getSelectedRecord()
-    // Returns the entire record object associated with the current value for this item
-    // (or null if no matching record exists in the PickList data).
-    // <P>
-    // Most commonly used for databound pickListItems to retrieve the values of other fields
-    // in the record.
-    //
-    // @return (object) record object
-    // @visibility external
-    //<
-    getSelectedRecord : function () {
-        if (this.pickList == null || this.pickList.destroyed) this.makePickList(false);
-        
-        // We can't just say this.pickList.getSelectedRecord(), since the
-        // value may not have been picked from a pickListClick -- instead force a selection
-        // that matches our item value then retrieve the selected value.
-        if (this.selectItemFromValue(this.getValue())) return this.pickList.getSelectedRecord();
-        return null;
-    },
-    
     // pickValue
     // This method is fired when a value is selected from the pick list.
     // Implementation will vary depending on the form item to which it is applied
