@@ -1,6 +1,6 @@
 /*
  * Isomorphic SmartClient
- * Version 8.0 (2010-05-18)
+ * Version SC_SNAPSHOT-2010-12-31 (2010-12-31)
  * Copyright(c) 1998 and beyond Isomorphic Software, Inc. All rights reserved.
  * "SmartClient" is a trademark of Isomorphic Software, Inc.
  *
@@ -78,10 +78,13 @@ isc.ValuesManager.addProperties({
     //<
     //members : null,    
     
-    //>	@attr	valuesManager.unknownErrorMessage	(string : "Invalid value" : [IRW])
-    // @include DynamicForm.unknownErrorMessage
+    //>	@attr	valuesManager.unknownErrorMessage	(string : null : [IRW])
+    // The error message for a failed validator that does not specify its own errorMessage.
+    // <P>
+    // If unset this value will be derived from the default 
+    // +link{dataBoundComponent.unknownErrorMessage} when the valuesManager is initialized.
     //<
-	unknownErrorMessage : "Invalid value"
+	unknownErrorMessage : null
     
     //> @attr valuesManager.disableValidation   (boolean : null : [IRWA])
     // @include DynamicForm.disableValidation
@@ -96,6 +99,10 @@ isc.ValuesManager.addMethods({
         // get a global ID so we can be called in the global scope
         this.ns.ClassFactory.addGlobalID(this);
         
+        if (this.unknownErrorMessage == null) {
+            this.unknownErrorMessage = isc.Canvas.getPrototype().unknownErrorMessage;
+        }
+        
         if (this.dataSource) this.bindToDataSource(this.dataSource);
 
         // Initialize this.values [and ensure it's a new object, so it can't be manipulated
@@ -1323,6 +1330,54 @@ isc.ValuesManager.addMethods({
     	return this._oldValues;
     },
     
+    //> @method valuesManager.getOldValues()
+    // @include dynamicForm.getOldValues()
+    //<
+    getOldValues : function () {
+        var oldValues = {};
+        isc.addProperties(oldValues, this._oldValues);
+        return oldValues;
+    },
+    
+    
+    //> @method valuesManager.getChangedValues()
+    // @include dynamicForm.getChangedValues()
+    // @see getOldValues()
+    // @visibility external
+    //<
+    
+    getChangedValues : function () {
+        var values = this.getValues(),
+            oldValues = this._oldValues, 
+            changed = false,
+            changedVals = {};
+        
+        if (!isc.isAn.Object(oldValues)) oldValues = {};
+        
+        for (var prop in values) {
+            // ignore functions
+            if (isc.isA.Function(values[prop])) continue;
+            
+            // Use compareValues to compare old and new values
+            // This will catch cases such as Dates where an '==' comparison is
+            // not sufficient.
+            // Note: If we have a form item use item.compareValues() in case it has been overridden
+            var item = this.getItem(prop);
+            if (item != null) {
+                changed = !item.compareValues(values[prop], oldValues[prop]);
+            } else {
+                changed = !isc.DynamicForm.compareValues(values[prop], oldValues[prop]);
+            }
+            // no need to keep going once we've found a difference
+            // unless we've been asked to return the changed values
+            if (changed) {
+                changedVals[prop] = values[prop];
+            }
+        }
+        
+        return changedVals
+    },
+
     _cloneValues : function (storedValues, values, dataPath) {
  
         var refPropName = "__ref";   
@@ -1498,7 +1553,8 @@ isc.ValuesManager.addMethods({
         if (memberDS != null && !fromDataPath && memberDS != this.getDataSource()) {
             this.logWarn("addMember(): mismatched DataSources; new member form " + member + 
                          " has dataSource: '" + memberDS.ID + 
-                         "', valuesManager has DataSource '" + this.getDataSource().ID + "'");
+                         "', valuesManager has DataSource " + 
+                         (this.getDataSource() != null ? "'"+this.getDataSource().ID+"'" : "[NONE]"));
         }
         
         // If any member forms are multipart, warn the developer - this implies that
@@ -1834,9 +1890,22 @@ isc.ValuesManager.registerStringMethods ({
     //                      &nbsp;&nbsp;<code>{fieldName:errors}</code><br>
     //                      Where the 'errors' object is either a single string or an array
     //                      of strings containing the error messages for the field.
+    // @return (boolean) false from this method to suppress that behavior
     // @visibility external
     //<
-    handleHiddenValidationErrors:"errors"
+    handleHiddenValidationErrors:"errors",
+    
+    //>	@method valuesManager.submitValues()
+    // Optional +link{stringMethod} to fire when +link{valuesManager.submit()} is called
+    // on this valuesManager (or any form included in this valuesManager).
+    // 
+    // @param	values    (object)        the valuesManager values
+    // @param	valuesManager      (ValuesManager)   the valuesManager being submitted
+    // @group submitting
+    // @see method:valuesManager.submit()
+    // @visibility external
+	//<
+    submitValues : "values,valuesManager"
 });
 
 //!<Deferred
