@@ -56,7 +56,20 @@ with (theWindow) {
         cornerSize:16,
         groupLabelBackgroundColor:"black",
         groupBorderCSS:"2px solid #D9D9D9"
-    })
+    });
+
+    if(isc.Browser.isIE && isc.Browser.version >= 7) {
+        isc.Canvas.setAllowExternalFilters(false);
+        isc.Canvas.setNeverUseFilters(true);
+        if(isc.Window) {
+          isc.Window.addProperties({
+                modalMaskOpacity:null,
+                modalMaskStyle:"normal"
+            });
+            isc.Window.changeDefaults("modalMaskDefaults", { src : "[SKIN]opacity.png" });
+        }
+    }
+
     isc.ScrollThumb.addProperties({
         capSize:6,
         vSrc:"[SKIN]vthumb.png",
