@@ -33,7 +33,9 @@ import com.smartgwt.client.widgets.tile.*;
 import com.smartgwt.client.widgets.tile.events.*;
 import com.smartgwt.client.widgets.grid.*;
 import com.smartgwt.client.widgets.grid.events.*;
+import com.smartgwt.client.widgets.chart.*;
 import com.smartgwt.client.widgets.layout.*;
+import com.smartgwt.client.widgets.layout.events.*;
 import com.smartgwt.client.widgets.menu.*;
 import com.smartgwt.client.widgets.tab.*;
 import com.smartgwt.client.widgets.toolbar.*;
@@ -58,7 +60,8 @@ import com.google.gwt.event.shared.HasHandlers;
 /**
  * Request sent to the server to initiate a  {@link com.smartgwt.client.docs.DataSourceOperations DataSource operation}. 
  * All properties which are legal on {@link com.smartgwt.client.rpc.RPCRequest} are legal, in addition to the properties
- * listed here.
+ * listed here. <P>
+ * @see com.smartgwt.client.rpc.RPCRequest
  */
 public class DSRequest extends RPCRequest {
 
@@ -175,6 +178,27 @@ public class DSRequest extends RPCRequest {
         return EnumUtil.getEnum(ExportFormat.values(), getAttribute("exportAs"));
     }
 
+    /**
+     * Only applies to request properties passed to {@link com.smartgwt.client.widgets.DataBoundComponent#exportClientData
+     * DataBoundComponent.exportClientData}. If specified this property contains an arbitrary set of data to be exported.
+     *
+     * @param exportData exportData Default value is null
+     */
+    public void setExportData(Record... exportData) {
+        setAttribute("exportData", exportData);
+    }
+
+    /**
+     * Only applies to request properties passed to {@link com.smartgwt.client.widgets.DataBoundComponent#exportClientData
+     * DataBoundComponent.exportClientData}. If specified this property contains an arbitrary set of data to be exported.
+     *
+     *
+     * @return Record
+     */
+    public Record[] getExportData()  {
+        return Record.convertToRecordArray(getAttributeAsJavaScriptObject("exportData"));
+    }
+
     /**
      * The character to use as a field-separator in CSV exports.  The default delimiter is comma.
      *
@@ -422,17 +446,19 @@ public class DSRequest extends RPCRequest {
      * <code>operationId</code> {@link com.smartgwt.client.data.OperationBinding#getOperationId on the operationBinding} which
      * will cause that operationBinding to be used for dsRequests containing a matching <code>operationId</code>. This allows
      * all the possible settings of an <code>operationBinding</code>, including {@link
-     * com.smartgwt.client.data.OperationBinding#getWsOperation wsOperation} or DMI settings, to be switched on a per-component
-     * or per-request basis.   <P> For example, by setting the <code>fetchOperation</code> on a particular ListGrid, you could
-     * cause it to invoke a different server method via DMI, different {@link
-     * com.smartgwt.client.data.OperationBinding#getDataURL dataURL} or different {@link
-     * com.smartgwt.client.data.OperationBinding#getWsOperation web service operation}. <P> The <code>operationId</code> can
-     * also be directly received by the server in order to affect behavior.  When using the Smart GWT Server,
-     * <code>operationId</code> can be accessed via dsRequest.getOperationId().  The {@link
+     * com.smartgwt.client.data.OperationBinding#getWsOperation wsOperation} or {@link
+     * com.smartgwt.client.data.DSRequest#getDmiOverview DMI} settings, to be switched on a per-component or per-request basis.
+     * <P> For example, by setting the <code>fetchOperation</code> on a particular ListGrid, you could cause it to invoke a
+     * different server method via DMI, different {@link com.smartgwt.client.data.OperationBinding#getDataURL dataURL} or
+     * different  {@link com.smartgwt.client.data.OperationBinding#getWsOperation web service operation}. <P> The
+     * <code>operationId</code> can also be directly received by the server in order to affect behavior.  When using the Smart
+     * GWT Server, <code>operationId</code> can be accessed via dsRequest.getOperationId().  The {@link
      * com.smartgwt.client.data.RestDataSource} will also send the <code>operationId</code> to the server as part of the {@link
      * com.smartgwt.client.data.RestDataSource#getMetaDataPrefix request metadata}.   <P> Note that if you {@link
      * com.smartgwt.client.data.DataSource#fetchData manually invoke} a DataSource operation, you can also specify operationId
-     * via the <code>requestProperties</code> parameter.
+     * via the <code>requestProperties</code> parameter. <P> Note that the <code>operationId</code> has special signficance in
+     * terms of whether two DSRequests are considered equivalent for caching and synchronization purposes - see {@link
+     * com.smartgwt.client.docs.DsRequestEquivalence}.
      *
      * @param operationId operationId Default value is null
      * @see com.smartgwt.client.docs.Operations Operations overview and related methods
@@ -453,17 +479,19 @@ public class DSRequest extends RPCRequest {
      * <code>operationId</code> {@link com.smartgwt.client.data.OperationBinding#getOperationId on the operationBinding} which
      * will cause that operationBinding to be used for dsRequests containing a matching <code>operationId</code>. This allows
      * all the possible settings of an <code>operationBinding</code>, including {@link
-     * com.smartgwt.client.data.OperationBinding#getWsOperation wsOperation} or DMI settings, to be switched on a per-component
-     * or per-request basis.   <P> For example, by setting the <code>fetchOperation</code> on a particular ListGrid, you could
-     * cause it to invoke a different server method via DMI, different {@link
-     * com.smartgwt.client.data.OperationBinding#getDataURL dataURL} or different {@link
-     * com.smartgwt.client.data.OperationBinding#getWsOperation web service operation}. <P> The <code>operationId</code> can
-     * also be directly received by the server in order to affect behavior.  When using the Smart GWT Server,
-     * <code>operationId</code> can be accessed via dsRequest.getOperationId().  The {@link
+     * com.smartgwt.client.data.OperationBinding#getWsOperation wsOperation} or {@link
+     * com.smartgwt.client.data.DSRequest#getDmiOverview DMI} settings, to be switched on a per-component or per-request basis.
+     * <P> For example, by setting the <code>fetchOperation</code> on a particular ListGrid, you could cause it to invoke a
+     * different server method via DMI, different {@link com.smartgwt.client.data.OperationBinding#getDataURL dataURL} or
+     * different  {@link com.smartgwt.client.data.OperationBinding#getWsOperation web service operation}. <P> The
+     * <code>operationId</code> can also be directly received by the server in order to affect behavior.  When using the Smart
+     * GWT Server, <code>operationId</code> can be accessed via dsRequest.getOperationId().  The {@link
      * com.smartgwt.client.data.RestDataSource} will also send the <code>operationId</code> to the server as part of the {@link
      * com.smartgwt.client.data.RestDataSource#getMetaDataPrefix request metadata}.   <P> Note that if you {@link
      * com.smartgwt.client.data.DataSource#fetchData manually invoke} a DataSource operation, you can also specify operationId
-     * via the <code>requestProperties</code> parameter.
+     * via the <code>requestProperties</code> parameter. <P> Note that the <code>operationId</code> has special signficance in
+     * terms of whether two DSRequests are considered equivalent for caching and synchronization purposes - see {@link
+     * com.smartgwt.client.docs.DsRequestEquivalence}.
      *
      *
      * @return String
@@ -674,6 +702,28 @@ public class DSRequest extends RPCRequest {
     // ***********************************************************        
 
 
+    
+    /**
+     * Set a custom attribute value on the DSRequest as an Object. Note that this method converts the Java primitive Object types, Dates and Maps to the underyling
+     * JavaScriptObject value. All other object types are set as Object type attributes and users are expected to call {@link #getAttributeAsObject(String)}
+     * in order to retrieve them.
+     * <P>
+     * These attributes are available for client-side use only - these attributes are not
+     * transmitted to the server.
+     * <P>
+     * Do not use setAttribute() to set any attribute for which there is a dedicated setter (do not
+     * setAttribute("data", <i>data</i>) for example).
+     * <P>
+     * If you are looking for a way to send additional data to the server, read
+     * {@link com.smartgwt.client.docs.DsRequestEquivalence,this overview} for the best approach.
+     *
+     * @param property the attribute name
+     * @param value the attribute value.
+     */
+    @Override
+    public void setAttribute(String property, Object value) {
+        super.setAttribute(property, value);
+    }
 
 /**
     * The list of field-names to export.  If provided, the field-list in the exported output is &#010 limited and sorted as per the list.&#010 <P>&#010 If exportFields is not provided, the exported output includes all visible fields &#010 from the DataSource (field.hidden=false), sorted in the order they're defined.
@@ -739,7 +789,19 @@ public class DSRequest extends RPCRequest {
      * @param sortSpecifiers  Default value is null
      */
     public void setSortBy(SortSpecifier[] sortSpecifiers) {
-        setAttribute("sortBy", sortSpecifiers);
+        if(sortSpecifiers != null && sortSpecifiers.length > 0) {
+            String sortBy = "";
+            for (int i = 0; i < sortSpecifiers.length; i++) {
+                SortSpecifier sortSpecifier = sortSpecifiers[i];
+                sortBy += ((sortSpecifier.getSortDirection() == SortDirection.DESCENDING ? "-" : "") + sortSpecifier.getField());
+                if(i < sortSpecifiers.length - 1) {
+                    sortBy += ",";
+                }
+            }
+            setAttribute("sortBy", sortBy);
+        } else {
+            setAttribute("sortBy", (String) null);
+        }
     }
 
     /**
@@ -749,12 +811,13 @@ public class DSRequest extends RPCRequest {
      */
     public native SortSpecifier[] getSortBy() /*-{
         var self = this.@com.smartgwt.client.core.DataClass::getJsObj()();
-        var sortSpec = self.sortBy;
-        if($wnd.isA.String(sortSpec)) {
-            return @com.smartgwt.client.data.SortSpecifier::convertToArray(Ljava/lang/String;)(sortSpec);
-        } else {
-            return @com.smartgwt.client.data.SortSpecifier::convertToArray(Lcom/google/gwt/core/client/JavaScriptObject;)(sortSpec);
+        var sortBy = self.sortBy;
+        if(sortBy == null || sortBy === undefined) return null;
+        //Unlike ListGrid.getSortBy(), DSRequest always stores sortBy as a String or an array of Strings, and not the SortSpecifier object / array.
+        if($wnd.isc.isA.Array(sortBy)) {
+            sortBy = sortBy.join(",")
         }
+        return @com.smartgwt.client.data.SortSpecifier::convertToArray(Ljava/lang/String;)(sortBy);
     }-*/;
 
     /**
@@ -806,9 +869,27 @@ public class DSRequest extends RPCRequest {
             throw new IllegalStateException("This method should only be called during FETCH operations");
         }
     }
+    
+    public static DSRequest[] convertToDSRequestArray(JavaScriptObject nativeArray) {
+        if (nativeArray == null) {
+            return new DSRequest[]{};
+        }
+        if (JSOHelper.isArray(nativeArray)) {
+            JavaScriptObject[] componentsj = JSOHelper.toArray(nativeArray);
+            DSRequest[] objects = new DSRequest[componentsj.length];
+            for (int i = 0; i < componentsj.length; i++) {
+                JavaScriptObject componentJS = componentsj[i];
+                objects[i] = DSRequest.getOrCreateRef(componentJS);
+            }
+            return objects;
+        } else {
+            DSRequest[] ret = new DSRequest[1];
+            ret[0] = DSRequest.getOrCreateRef(nativeArray);
+            return ret;
+        }
+    }
 
 }
 
 
 
-
