@@ -23,13 +23,12 @@ public interface AuthenticationService extends RemoteService {
 	public boolean authenticate(String login, String passwd);
 	
 	/**
-	 * Supposed to return user group or null if no user is logged.
-	 * As we only have admin and guest mode, whe return a random string
-	 * if admin is logged and null if not.
+	 * Returns logged user login and type in the form <login>:<type>.
+	 * Type can be user or admin.
 	 * 
 	 * @return
 	 */
-	public String getUserAcces();
+	public String getUserAccess();
 	
 	/**
 	 * Removes user from session.
