@@ -17,7 +17,7 @@ public interface AuthenticationServiceAsync {
 	void authenticate(String login, String passwd,
 			AsyncCallback<Boolean> callback);
 
-	void getUserAcces(AsyncCallback<String> callback);
+	void getUserAccess(AsyncCallback<String> callback);
 
 	void logout(AsyncCallback<Void> callback);
 }
