@@ -10,17 +10,17 @@ public interface BankEditingServiceAsync {
 	
 	public static final BankEditingServiceAsync INSTANCE = GWT.create(BankEditingService.class);
 
-	void getBankProperties(String bankName, String type, Collection<String> included, AsyncCallback<Map<String, String>> callback);
+	void getBankProperties(String login, String bankName, String type, Collection<String> included, AsyncCallback<Map<String, String>> callback);
 
-	void getBankList(AsyncCallback<Collection<String>> callback);
+	void getBankList(String login, AsyncCallback<Collection<String>> callback);
 
-	void getIncludedProperties(String bankName, Collection<String> included,
+	void getIncludedProperties(String login, String bankName, Collection<String> included,
 			AsyncCallback<Map<String, Map<String, String>>> callback);
 
 	void getRemoteBankList(String url, AsyncCallback<Collection<String>> callback);
 
-	void save(String type, String bankName, String newName, Map<String, String> newProps, boolean runUpdate,
-			AsyncCallback<Void> callback);
+	void save(String login, String type, String bankName, String newName, Map<String, String> newProps, boolean runUpdate,
+			AsyncCallback<Integer> callback);
 
 	void getEmptyBank(String pattern,
 			AsyncCallback<Map<String,  Map<String, String>>> callback);
@@ -31,11 +31,11 @@ public interface BankEditingServiceAsync {
 
 	void getPrefixedKey(String key, AsyncCallback<String> callback);
 
-	void getIncludePropertiesList(AsyncCallback<Collection<String>> callback);
+	void getIncludePropertiesList(String user, AsyncCallback<Collection<String>> callback);
 
 	void getProcessList(AsyncCallback<Collection<String>> callback);
 
 	void getProcessDescription(String process, AsyncCallback<String> callback);
 
-	void propertiesFileExists(String fileName, AsyncCallback<Boolean> callback);
+	void propertiesFileExists(String login, String fileName, AsyncCallback<Boolean> callback);
 }
