@@ -12,9 +12,9 @@ public interface BankStatServiceAsync {
 	
 	void banksStatus(AsyncCallback<Map<String, Integer>> callback);
 
-	void getBankStats(String bankName,
+	void getBankStats(String bankName, int max,
 			AsyncCallback<List<Map<String, String>>> callback);
 
-	void getBanksInfo(AsyncCallback<List<Map<String, String>>> callback);
+	void getBanksInfo(String userKey, AsyncCallback<List<Map<String, String>>> callback);
 
 }
