@@ -1,24 +1,32 @@
 package org.inria.bmajwatcher.client.ui.scheduler;
 
 
+import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
 import org.inria.bmajwatcher.client.i18n.BmajWatcherConstants;
+import org.inria.bmajwatcher.client.services.BiomajLauncherServiceAsync;
 import org.inria.bmajwatcher.client.services.SchedulingServiceAsync;
+import org.inria.bmajwatcher.client.services.UserManagerServiceAsync;
 import org.inria.bmajwatcher.client.ui.BankGridDataSource;
+import org.inria.bmajwatcher.client.ui.confirmation.MessagePanel;
+import org.inria.bmajwatcher.client.ui.styles.MainCss;
 import org.inria.bmajwatcher.client.ui.styles.Resources;
 
 import com.google.gwt.user.client.rpc.AsyncCallback;
 import com.google.gwt.user.client.ui.Button;
-import com.google.gwt.user.client.ui.HasAlignment;
+import com.google.gwt.user.client.ui.HasHorizontalAlignment;
+import com.google.gwt.user.client.ui.HasVerticalAlignment;
 import com.google.gwt.user.client.ui.HorizontalPanel;
 import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Panel;
 import com.google.gwt.user.client.ui.PushButton;
+import com.google.gwt.user.client.ui.ScrollPanel;
 import com.smartgwt.client.types.Alignment;
 import com.smartgwt.client.types.MultipleAppearance;
+import com.smartgwt.client.types.Overflow;
 import com.smartgwt.client.types.TimeFormatter;
 import com.smartgwt.client.types.VisibilityMode;
 import com.smartgwt.client.util.BooleanCallback;
@@ -49,10 +57,8 @@ import com.smartgwt.client.widgets.grid.ListGridRecord;
 import com.smartgwt.client.widgets.grid.events.CellContextClickEvent;
 import com.smartgwt.client.widgets.grid.events.CellContextClickHandler;
 import com.smartgwt.client.widgets.layout.HLayout;
-import com.smartgwt.client.widgets.layout.Layout;
 import com.smartgwt.client.widgets.layout.SectionStack;
 import com.smartgwt.client.widgets.layout.SectionStackSection;
-import com.smartgwt.client.widgets.layout.VLayout;
 import com.smartgwt.client.widgets.menu.Menu;
 import com.smartgwt.client.widgets.menu.MenuItem;
 import com.smartgwt.client.widgets.menu.events.MenuItemClickEvent;
@@ -65,24 +71,34 @@ import com.smartgwt.client.widgets.menu.events.MenuItemClickEvent;
  */
 public class Scheduler {
 
-	private VLayout layout;
 	private SectionStack tasks;
 	private ListGrid mainGrid;
 	private HorizontalPanel buttonPanel;
+	private ScrollPanel container;
 	
 	private static final short NEW_TASK = 0;
 	private static final short EDIT_TASK = 1;
 	
-	public Scheduler(ListGrid grid) {
+	private final int MIN_HEIGHT = 446;
+	
+	private String login;
+	
+	private SchedulingServiceAsync schedulerService = SchedulingServiceAsync.INSTANCE;
+	private MainCss css = Resources.INSTANCE.mainCss();
+	private PushButton newTask;
+	
+	
+	public Scheduler(ListGrid grid, String userKey) {
 		
 		mainGrid = grid;
 		
 		tasks = new SectionStack();
 		tasks.setVisibilityMode(VisibilityMode.MULTIPLE);
 		tasks.setWidth(270);
-		tasks.setHeight(446);
+		tasks.setHeight(MIN_HEIGHT);
+		tasks.setOverflow(Overflow.AUTO);
 		
-		PushButton newTask = new PushButton(new Image("images/appointment-new.png"));
+		newTask = new PushButton(new Image("images/appointment-new.png"));
 		newTask.setTitle(BmajWatcherConstants.INSTANCE.newTask());
 		newTask.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
 			
@@ -92,47 +108,204 @@ public class Scheduler {
 			}
 		});
 		
-		layout = new VLayout();
-		layout.setHeight(446);
-		layout.setMembersMargin(5);
-		
-		PushButton showCal = new PushButton(new Image("images/view-calendar.png"));
+		final PushButton showCal = new PushButton(new Image("images/view-calendar.png"));
 		showCal.setTitle("Calendar view");
-		showCal.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
+		
+		buttonPanel = new HorizontalPanel();
+		buttonPanel.setHeight("35px");
+		buttonPanel.add(newTask);
+		buttonPanel.add(showCal);
+		buttonPanel.setCellHorizontalAlignment(newTask, HasHorizontalAlignment.ALIGN_RIGHT);
+		buttonPanel.setCellHorizontalAlignment(showCal, HasHorizontalAlignment.ALIGN_RIGHT);
+		
+		final HorizontalPanel schedButtons = new HorizontalPanel();
+		schedButtons.addStyleName(css.schedulerButtons());
+		schedButtons.setSpacing(2);
+
+		
+		// Refresh
+		final Image imgRefreshSched = new Image("images/arrow-refresh.png");
+		imgRefreshSched.setTitle("Refresh jobs");
+		imgRefreshSched.addStyleName(css.imgPointer());
+		imgRefreshSched.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
 			
 			@Override
 			public void onClick(com.google.gwt.event.dom.client.ClickEvent event) {
-				TaskViewer viewer = new TaskViewer();
-				viewer.show();
+				int size = tasks.getSections().length;
+				for (int i = 0; i < size; i++) {
+					tasks.removeSection(0);
+				}
+				loadJobs();
 			}
 		});
-		buttonPanel = new HorizontalPanel();
-		buttonPanel.setHeight("35px");
-//		buttonPanel.setSize("274px", "40px");
-		buttonPanel.add(newTask);
-		buttonPanel.add(showCal);
-		buttonPanel.setCellHorizontalAlignment(newTask, HasAlignment.ALIGN_RIGHT);
-		buttonPanel.setCellHorizontalAlignment(showCal, HasAlignment.ALIGN_RIGHT);
 
-		layout.addMember(tasks);
+		schedButtons.add(imgRefreshSched);
 		
-		loadJobs();
 		
-		if (tasks.getSections().length == 0) {
-			tasks.setVisible(false);
-			layout.setHeight(30);
-		}
+		buttonPanel.add(schedButtons);
+		buttonPanel.setCellWidth(schedButtons, "210px");
+		buttonPanel.setCellHorizontalAlignment(schedButtons, HasHorizontalAlignment.ALIGN_RIGHT);
+		buttonPanel.setCellVerticalAlignment(schedButtons, HasVerticalAlignment.ALIGN_MIDDLE);
+		
+
+		container = new ScrollPanel(tasks);
+		container.setSize("270px", "446px");
+		
+		
+		UserManagerServiceAsync.INSTANCE.getUserLoginFromKey(userKey, new AsyncCallback<String>() {
+			
+			@Override
+			public void onSuccess(String result) {
+				if (result != null) {
+					login = result;
+					
+					
+					UserManagerServiceAsync.INSTANCE.getAdminLogin(new AsyncCallback<String>() {
+						
+						@Override
+						public void onSuccess(final String adminLogin) {
+							final Image imgStartSched = new Image("images/normal-update.png");
+							imgStartSched.setTitle("Start scheduler");
+							imgStartSched.addStyleName(css.imgPointer());
+							final Image imgStopSched = new Image("images/stop.png");
+							imgStopSched.setTitle("Put scheduler on stand-by");
+							imgStopSched.addStyleName(css.imgPointer());
+							
+							/*
+							 *  Admin can start/stop scheduler
+							 */
+							if (adminLogin.equals(login)) {
+								
+								// Start
+								imgStartSched.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
+									
+									@Override
+									public void onClick(com.google.gwt.event.dom.client.ClickEvent event) {
+										schedulerService.start(new AsyncCallback<Boolean>() {
+											
+											@Override
+											public void onSuccess(Boolean result) {
+												if (result) {
+													imgStartSched.setVisible(false);
+													imgStopSched.setVisible(true);
+													tasks.setDisabled(false);
+													newTask.setEnabled(true);
+												} else {
+													SC.warn("Could not start the scheduler");
+												}
+											}
+											
+											@Override
+											public void onFailure(Throwable caught) {
+												SC.warn("RPC error");
+											}
+										});
+									}
+								});
+								
+								// Stop
+								imgStopSched.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
+									
+									@Override
+									public void onClick(com.google.gwt.event.dom.client.ClickEvent event) {
+										schedulerService.stop(new AsyncCallback<Boolean>() {
+											
+											@Override
+											public void onSuccess(Boolean result) {
+												if (result) {
+													imgStartSched.setVisible(true);
+													imgStopSched.setVisible(false);
+													tasks.setDisabled(true);
+													newTask.setEnabled(false);
+												} else {
+													SC.warn("Could not stop the scheduler");
+												}
+											}
+											
+											@Override
+											public void onFailure(Throwable caught) {
+												SC.warn("RPC failure");
+											}
+										});
+									}
+								});
+								
+								schedButtons.add(imgStartSched);
+								schedButtons.add(imgStopSched);
+								
+							}
+							
+							// Check scheduler status
+							schedulerService.isStarted(new AsyncCallback<Boolean>() {
+										
+								@Override
+								public void onSuccess(Boolean result) {
+									if (result) {
+										if (login.equals(adminLogin)) { // if admin
+											imgStartSched.setVisible(false);
+											imgStopSched.setVisible(true);
+										}
+										newTask.setEnabled(true); 
+										tasks.setDisabled(false);
+									} else {
+										if (login.equals(adminLogin)) { // if admin
+											imgStartSched.setVisible(true);
+											imgStopSched.setVisible(false);
+										}
+										newTask.setEnabled(false);
+										tasks.setDisabled(true);
+									}
+								}
+								
+								@Override
+								public void onFailure(Throwable caught) {
+									SC.warn("RPC failure. Could not retrieve scheduler status");
+								}
+							});
+						}
+						
+						@Override
+						public void onFailure(Throwable caught) {
+							SC.warn("RPC failure. Could not get admin login");
+						}
+					});
+					
+					
+					
+					loadJobs();
+					
+					showCal.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
+						
+						@Override
+						public void onClick(com.google.gwt.event.dom.client.ClickEvent event) {
+							TaskViewer viewer = new TaskViewer(login);
+							viewer.show();
+						}
+					});
+				}
+			}
+			
+			@Override
+			public void onFailure(Throwable caught) {
+				SC.warn("RPC failure");
+			}
+		});
 	}
 	
 	private void loadJobs() {
-		SchedulingServiceAsync.INSTANCE.getJobs(new AsyncCallback<Map<String,List<String>>>() {
+		SchedData.getInstance().clear();
+		schedulerService.getJobs(login, new AsyncCallback<Map<String,List<String>>>() {
 			
 			@Override
 			public void onSuccess(Map<String, List<String>> result) {
 				Set<String> set = result.keySet();
 				for (String str : set) {
 					String[] tab = str.split(":");
-					createTaskTab(SchedData.getInstance().addTask(tab[0], tab[1]), result.get(str));
+					createTaskTab(SchedData.getInstance().addTask(login, tab[0], tab[1]), result.get(str));
+				}
+				resizeTasks();
+				if (tasks.getSections().length == 0) {
+					tasks.setVisible(false);
 				}
 			}
 			
@@ -148,7 +321,7 @@ public class Scheduler {
 	 *  
 	 * @return
 	 */
-	private ListGrid getGrid(final String id, List<String> banks) {
+	private ListGrid getGrid(final String id, final List<String> banks) {
 
 		final ListGrid bankList = new ListGrid();
 		// testGrid.setWidth(250);
@@ -161,11 +334,50 @@ public class Scheduler {
 
 		
 		if (banks != null) {
-			for (String bank : banks) {
-				ListGridRecord rec = new ListGridRecord();
-				rec.setAttribute(BankGridDataSource.NAME, bank);
-				bankList.addData(rec);
-			}
+			UserManagerServiceAsync.INSTANCE.getAdminLogin(new AsyncCallback<String>() {
+				
+				@Override
+				public void onSuccess(String result) {
+					
+					boolean admin = result.equals(login);
+					// As users groups can be changed or bank can be deleted,
+					// need to get the banks list logged user has access to.
+					List<String> validBanks = new ArrayList<String>();
+					for (ListGridRecord rec : mainGrid.getRecords()) {
+						if (rec.getAttribute(BankGridDataSource.GROUP).equalsIgnoreCase("true") || admin)
+							validBanks.add(rec.getAttribute(BankGridDataSource.NAME));
+					}
+
+					List<String> banksToRemoveFromTask = new ArrayList<String>();
+					for (String bank : banks) {
+						if (!validBanks.contains(bank)) {
+							banksToRemoveFromTask.add(bank);
+						} else {
+							ListGridRecord rec = new ListGridRecord();
+							rec.setAttribute(BankGridDataSource.NAME, bank);
+							bankList.addData(rec);
+						}
+					}
+					
+					schedulerService.removeBanksFromTrigger(banksToRemoveFromTask.toArray(new String[banksToRemoveFromTask.size()]),
+							SchedData.getInstance().getName(id), new AsyncCallback<Void>() {
+								
+								@Override
+								public void onSuccess(Void result) {
+								}
+								
+								@Override
+								public void onFailure(Throwable caught) {
+									SC.warn("RPC failure");
+								}
+							});
+				}
+				
+				@Override
+				public void onFailure(Throwable caught) {
+					SC.warn("RPC failure");
+				}
+			});
 		}
 		
 		/*
@@ -211,7 +423,7 @@ public class Scheduler {
 				if (exists)
 					event.cancel();
 				else {
-					SchedulingServiceAsync.INSTANCE.addBanksToTrigger(banks,
+					schedulerService.addBanksToTrigger(banks,
 							SchedData.getInstance().getName(id), new AsyncCallback<Void>() {
 						
 						@Override
@@ -277,7 +489,7 @@ public class Scheduler {
 				/*
 				 * Remove bank
 				 */
-				SchedulingServiceAsync.INSTANCE.removeBanksFromTrigger(banks, taskLabel, new AsyncCallback<Void>() {
+				schedulerService.removeBanksFromTrigger(banks, taskLabel, new AsyncCallback<Void>() {
 					
 					@Override
 					public void onSuccess(Void result) {
@@ -303,11 +515,65 @@ public class Scheduler {
 	 * at a given time.
 	 * 
 	 */
-	private void createTaskTab(final String id, List<String> banks) {
+	private void createTaskTab(final String id, final List<String> banks) {
 		
 		final SectionStackSection task = new SectionStackSection(SchedData.getInstance().getName(id));
 		task.setID(task.toString());
-		task.setItems(getGrid(id, banks));
+		final ListGrid content = getGrid(id, banks);
+		task.setItems(content);
+		
+		
+		/*
+		 * Start all banks
+		 */
+		ImgButton allUpdatesBtn = new ImgButton();
+		allUpdatesBtn.setSrc("update_all.png");
+		allUpdatesBtn.setSize(16);
+		allUpdatesBtn.setShowRollOver(false);
+		allUpdatesBtn.setShowDown(false);
+		allUpdatesBtn.setTooltip("Update all banks of this schedule");
+		allUpdatesBtn.addClickHandler(new ClickHandler() {
+			
+			@Override
+			public void onClick(ClickEvent event) {
+				
+				if (content.getRecords().length > 0) {
+					
+					final StringBuilder toUpdate = new StringBuilder();
+					for (ListGridRecord bank : content.getRecords()) {
+						toUpdate.append(bank.getAttribute(BankGridDataSource.NAME) + " ");
+					}
+					
+					System.out.println("TO update : " + toUpdate.toString());
+					
+					SC.confirm("Confirm update start ?", new BooleanCallback() {
+						
+						@Override
+						public void execute(Boolean value) {
+							if (value) {
+								
+								BiomajLauncherServiceAsync.INSTANCE.startUpdate(toUpdate.toString().trim(), new AsyncCallback<Boolean>() {
+									
+									@Override
+									public void onSuccess(Boolean result) {
+										if (result) {
+											MessagePanel.showMessage("Updates started");
+										} else {
+											SC.warn("Updates could no be started");
+										}
+									}
+									
+									@Override
+									public void onFailure(Throwable caught) {
+										SC.warn("RPC failure");
+									}
+								});
+							}
+						}
+					});
+				}
+			}
+		});
 		
 		/*
 		 * Edit task
@@ -347,10 +613,15 @@ public class Scheduler {
 					public void execute(Boolean value) {
 						if (value) {
 							
-							SchedulingServiceAsync.INSTANCE.deleteTrigger(SchedData.getInstance().getName(id), new AsyncCallback<Void>() {
+							schedulerService.deleteTrigger(SchedData.getInstance().getName(id), new AsyncCallback<Void>() {
 								
 								@Override
 								public void onSuccess(Void result) {
+									SchedData.getInstance().deleteTask(id);
+									tasks.removeSection(task.toString());
+									resizeTasks();
+									if (tasks.getSections().length == 0)
+										tasks.setVisible(false);
 								}
 								
 								@Override
@@ -358,25 +629,29 @@ public class Scheduler {
 									SC.say("Failure");
 								}
 							});
-							
-							SchedData.getInstance().deleteTask(id);
-							tasks.removeSection(task.toString());
-							if (tasks.getSections().length == 0)
-								tasks.setVisible(false);
 						}
 					}
 				});
 			}
 		});
 		
-		task.setControls(editBtn, removeBtn);
+		task.setControls(allUpdatesBtn, editBtn, removeBtn);
 		task.setExpanded(true);
 
 		tasks.addSection(task);
 		tasks.setVisible(true);
 		
-//		tasks.setMargin(5);
-//		tasks.setBorder("none");
+	}
+	
+	private void resizeTasks() {
+		int height = tasks.getSections().length * 80;
+		if (height > MIN_HEIGHT) {
+			tasks.setWidth(254);
+			tasks.setHeight(height);
+		} else {
+			tasks.setWidth(270);
+			tasks.setHeight(MIN_HEIGHT);
+		}
 	}
 	
 	private void editTaskTab(String id, String label, String expr) {
@@ -417,7 +692,7 @@ public class Scheduler {
 		final Window winModal = new Window();
 //		winModal.setStyleName(CSSResources.INSTANCE.mainCss().taskEditor());
 		winModal.setWidth(500);
-		winModal.setHeight(230);
+		winModal.setHeight(350);
 		winModal.setTitle(BmajWatcherConstants.INSTANCE.editTask());
 		winModal.setShowMinimizeButton(false);
 		winModal.setIsModal(true);
@@ -437,6 +712,7 @@ public class Scheduler {
 		expr.setWidth(350);
 		
 		final CheckboxItem assistedCheck = new CheckboxItem("assistedCheck", "Assisted cron creation");
+		assistedCheck.setValue(true);
 		assistedCheck.addChangedHandler(new ChangedHandler() {
 			
 			@Override
@@ -535,8 +811,8 @@ public class Scheduler {
 		cancelTask.setWidth("60px");
 		HLayout btnLayout = new HLayout();
 		btnLayout.setPadding(10);
-		btnLayout.addMember(cancelTask);
 		btnLayout.addMember(createTask);
+		btnLayout.addMember(cancelTask);
 		btnLayout.setAlign(Alignment.CENTER);
 
 		cancelTask.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
@@ -565,7 +841,7 @@ public class Scheduler {
 						 * Modify cron
 						 */
 						if (!newCron.equals(exp)) {
-							SchedulingServiceAsync.INSTANCE.setCron(lbl, newCron, new AsyncCallback<Boolean>() {
+							schedulerService.setCron(lbl, newCron, new AsyncCallback<Boolean>() {
 								
 								@Override
 								public void onSuccess(Boolean result) {
@@ -587,13 +863,16 @@ public class Scheduler {
 						 * Modify name
 						 */
 						if (!newLabel.equals(lbl)) {
-							if (!SchedData.getInstance().jobExists(newLabel)) {
+							if (!SchedData.getInstance().jobExists(login, newLabel)) {
 								SchedData.getInstance().changeName(dataId, newLabel);
-								SchedulingServiceAsync.INSTANCE.renameTrigger(lbl, newLabel, new AsyncCallback<Void>() {
+								schedulerService.renameTrigger(lbl, newLabel, new AsyncCallback<Boolean>() {
 									
 									@Override
-									public void onSuccess(Void result) {
-										winModal.hide();
+									public void onSuccess(Boolean result) {
+										if (!result)
+											SC.warn("Task name already exists");
+										else
+											winModal.hide();
 									}
 									
 									@Override
@@ -610,16 +889,19 @@ public class Scheduler {
 					 * Create task
 					 */
 					} else {
-						if (!SchedData.getInstance().jobExists(newLabel)) {
-							SchedulingServiceAsync.INSTANCE.createTrigger(newLabel, newCron, new AsyncCallback<Boolean>() {
+						if (!SchedData.getInstance().jobExists(login, newLabel)) {
+							schedulerService.createTrigger(login, newLabel, newCron, new AsyncCallback<Integer>() {
 								
 								@Override
-								public void onSuccess(Boolean result) {
-									if (result) {
+								public void onSuccess(Integer result) {
+									if (result > 0) {
 										winModal.hide();
-										createTaskTab(SchedData.getInstance().addTask(newLabel, newCron), null);
-									} else
+										createTaskTab(SchedData.getInstance().addTask(login, newLabel, newCron), null);
+										resizeTasks();
+									} else if (result < 0)
 										SC.warn("Cron syntax error");
+									else
+										SC.warn("Task name already exists");
 								}
 								
 								@Override
@@ -641,7 +923,6 @@ public class Scheduler {
 		assistedCanvas.setWidth100();
 		assistedCanvas.addChild(assistedForm);
 		winModal.addItem(assistedCanvas);
-		assistedCanvas.setVisible(false);
 		winModal.addItem(manualForm);
 		winModal.addItem(btnLayout);
 		
@@ -675,12 +956,12 @@ public class Scheduler {
 	}
 	
 	/**
-	 * Returns the layout contains the scheduler elements.
+	 * Returns the panel that contains the scheduler elements.
 	 *  
 	 * @return
 	 */
-	public Layout getLayout() {
-		return layout;
+	public Panel getContainer() {
+		return container;
 	}
 	
 	public Panel getButtonPanel() {
