@@ -53,10 +53,12 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 	private Map<String, CronTrigger> waitingTriggers = new HashMap<String, CronTrigger>();
 	private static final String DEFAULT_GROUP = "default";
 	
-	private Map<Date, Map<String, List<String>>> sessionLogs = new HashMap<Date, Map<String,List<String>>>();
-	private Date firstDayOfMonth = null; 
+	private Map<String, Map<Date, Map<String, List<String>>>> sessionLogs = new HashMap<String, Map<Date, Map<String,List<String>>>>();
+	private static final int CACHE_DURATION = 30000; // How long cache is kept in millis
+	private static Map<String, Long> lastCallToSessionLogs = new HashMap<String, Long>(); // key is "<year>-<month>", value is last call date in millis
+	
+	private Map<String, Date> firstDayOfMonth = new HashMap<String, Date>();
 	
-	private static long MILLIS_IN_DAY = 1000L * 3600L * 24L;
 	private static long MILLIS_IN_MONTH = 1000L * 3600L * 24L * 31L;
 	
 	
@@ -66,7 +68,7 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 		
 		System.setProperty("BIOMAJ_ROOT", getServletContext().getInitParameter("BIOMAJ_ROOT")); // Used in biomaj.jar
 		
-		serializer = JobSerializer.getInstance(config.getServletContext().getRealPath("/"));
+		serializer = JobSerializer.getInstance(getServletContext().getInitParameter("JOBS_LOCATION"));
 		
 		loadJobs(serializer.getJobs());
 		/*
@@ -102,35 +104,39 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 			log.error("Scheduler loading error", e);
 		}
 		
-		List<?> jobs = root.getChildren(JobSerializer.JOB);
-		
-		for (Object job : jobs) {
-			Element current = (Element) job;
-			List<?> banks = current.getChildren();
-			CronTrigger trigger = null;
-			try {
-				trigger = new CronTrigger(current.getAttributeValue(JobSerializer.JOB_NAME),
-						DEFAULT_GROUP, current.getAttributeValue(JobSerializer.JOB_CRON));
-				trigger.setMisfireInstruction(CronTrigger.MISFIRE_INSTRUCTION_DO_NOTHING);
-				if (banks.size() == 0) {	
-					waitingTriggers.put(current.getAttributeValue(JobSerializer.JOB_NAME), trigger);
-				} else {
-					JobDetail jobDetail = new JobDetail(current.getAttributeValue(JobSerializer.JOB_NAME), DEFAULT_GROUP, UpdateJob.class);
-					String description = "";
-					for (Object bank : banks) {
-						Element elt = (Element) bank;
-						description += " " + elt.getAttributeValue(JobSerializer.BANK_NAME);
-					}
-					jobDetail.setDescription(description.trim());
-					
-					try {
-						sched.scheduleJob(jobDetail, trigger);
-					} catch (SchedulerException e) {
-						log.error("Scheduling error", e);
+		List<?> users = root.getChildren(JobSerializer.USER);
+		for (Object user : users) {
+			Element usr = (Element) user;
+			List<?> jobs = usr.getChildren(JobSerializer.JOB);
+			for (Object job : jobs) {
+				Element current = (Element) job;
+				List<?> banks = current.getChildren();
+				CronTrigger trigger = null;
+				try {
+					trigger = new CronTrigger(current.getAttributeValue(JobSerializer.JOB_NAME),
+							DEFAULT_GROUP, current.getAttributeValue(JobSerializer.JOB_CRON));
+					trigger.setMisfireInstruction(CronTrigger.MISFIRE_INSTRUCTION_DO_NOTHING);
+					if (banks.size() == 0) {	
+						waitingTriggers.put(current.getAttributeValue(JobSerializer.JOB_NAME), trigger);
+					} else {
+						JobDetail jobDetail = new JobDetail(current.getAttributeValue(JobSerializer.JOB_NAME),
+								DEFAULT_GROUP, UpdateJob.class);
+						String description = "";
+						for (Object bank : banks) {
+							Element elt = (Element) bank;
+							description += " " + elt.getAttributeValue(JobSerializer.BANK_NAME);
+						}
+						jobDetail.setDescription(description.trim());
+						
+						try {
+							sched.scheduleJob(jobDetail, trigger);
+						} catch (SchedulerException e) {
+							log.error("Scheduling error", e);
+						}
 					}
+				} catch (ParseException ex) {
+					log.error("Invalid cron syntax", ex);
 				}
-			} catch (ParseException ex) {
-				log.error("Invalid cron syntax", ex);
 			}
 		}
 	}
@@ -169,8 +175,8 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 	}
 
 	@Override
-	public boolean createTrigger(String label, String cron) {
-		log.debug("Create trigger : " + label + "," + cron);
+	public int createTrigger(String user, String label, String cron) {
+		log.debug("Create trigger : " + label + "," + cron + " for user '" + user  + "'");
 		
 		/*
 		 * A trigger can only be registered to the scheduler with
@@ -179,19 +185,25 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 		 * addBankToTrigger is called.
 		 */
 		
-		try {
-			CronTrigger ct = new CronTrigger(label, DEFAULT_GROUP, cron);
-			ct.setMisfireInstruction(CronTrigger.MISFIRE_INSTRUCTION_DO_NOTHING);
-			waitingTriggers.put(label, ct);
-			
-			serializer.createJob(label, cron);
-			
-			return true;
-		} catch (ParseException ex) {
-			log.error("Cron syntax error", ex);
+		if (!serializer.jobExists(label)) {
+		
+			try {
+				CronTrigger ct = new CronTrigger(label, DEFAULT_GROUP, cron);
+				ct.setMisfireInstruction(CronTrigger.MISFIRE_INSTRUCTION_DO_NOTHING);
+				waitingTriggers.put(label, ct);
+				
+				serializer.createJob(user, label, cron);
+				// OK
+				return 1;
+			} catch (ParseException ex) {
+				log.error("Cron syntax error", ex);
+				// Syntax error
+				return -1;
+			}
 		}
 		
-		return false;
+		// Job exists
+		return 0;
 	}
 
 	@Override
@@ -212,6 +224,9 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 
 	@Override
 	public void removeBanksFromTrigger(String[] bankNames, String triggerName) {
+		if (bankNames.length == 0)
+			return;
+		
 		for (String bankName : bankNames)
 			log.debug("Remove bank '" + bankName + "' from trigger : " + triggerName);
 		
@@ -251,32 +266,38 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 	}
 
 	@Override
-	public void renameTrigger(String oldName, String newName) {
+	public boolean renameTrigger(String oldName, String newName) {
 		log.debug("Rename trigger : " + oldName + " -> " + newName);
 		
-		serializer.renameJob(oldName, newName);
 		
-		
-		if (waitingTriggers.containsKey(oldName)) {
-			CronTrigger trg = waitingTriggers.get(oldName);
-			trg.setName(newName);
-			waitingTriggers.put(newName, trg);
-			waitingTriggers.remove(oldName);
-		} else {
-			try {
-				String expr = ((CronTrigger) sched.getTrigger(oldName, DEFAULT_GROUP)).getCronExpression();
-				CronTrigger trig = new CronTrigger(newName, DEFAULT_GROUP, expr);
-				trig.setMisfireInstruction(CronTrigger.MISFIRE_INSTRUCTION_DO_NOTHING);
-				JobDetail detail = (JobDetail) sched.getJobDetail(oldName, DEFAULT_GROUP).clone();
-				detail.setName(newName);
-				sched.unscheduleJob(oldName, DEFAULT_GROUP);
-				sched.scheduleJob(detail, trig);
-			} catch (SchedulerException e) {
-				log.error("Scheduling error", e);
-			} catch (ParseException e) {
-				log.error("Parsing error", e);
+		if (serializer.renameJob(oldName, newName)) {
+			
+			
+			if (waitingTriggers.containsKey(oldName)) {
+				CronTrigger trg = waitingTriggers.get(oldName);
+				trg.setName(newName);
+				waitingTriggers.put(newName, trg);
+				waitingTriggers.remove(oldName);
+			} else {
+				try {
+					String expr = ((CronTrigger) sched.getTrigger(oldName, DEFAULT_GROUP)).getCronExpression();
+					CronTrigger trig = new CronTrigger(newName, DEFAULT_GROUP, expr);
+					trig.setMisfireInstruction(CronTrigger.MISFIRE_INSTRUCTION_DO_NOTHING);
+					JobDetail detail = (JobDetail) sched.getJobDetail(oldName, DEFAULT_GROUP).clone();
+					detail.setName(newName);
+					sched.unscheduleJob(oldName, DEFAULT_GROUP);
+					sched.scheduleJob(detail, trig);
+				} catch (SchedulerException e) {
+					log.error("Scheduling error", e);
+				} catch (ParseException e) {
+					log.error("Parsing error", e);
+				}
 			}
+			
+			return true;
 		}
+		
+		return false;
 	}
 
 	@Override
@@ -313,14 +334,16 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 	}
 
 	@Override
-	public Map<String, List<String>> getJobs() {
-		return serializer.getJobsAsMap();
+	public Map<String, List<String>> getJobs(String user) {
+		return serializer.getJobsAsMap(user);
 	}
 	
 	@Override
-	public Map<Date, String> getJobsDates(Date current, int range) {
+	public Map<Date, String> getJobsDates(String user, Date current, int range) {
 		
-		Map<String, List<String>> jobs = serializer.getJobsAsMap();
+		boolean admin = UserManagerServiceImpl.adminLogin().equals(user);
+		
+		Map<String, List<String>> jobs = serializer.getJobsAsMap(user);
 		Set<String> keys = jobs.keySet();
 		Map<Date, String> res = new HashMap<Date, String>();
 		for (String key : keys) {
@@ -340,6 +363,9 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 			}
 		}
 		
+		
+		
+		getMonthLogs(current, user, admin);
 		// Look for previous sessions logs
 		if (range == 0) { // Month view
 			Calendar cal = Calendar.getInstance();
@@ -351,7 +377,7 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 			
 			Date today = new Date();
 			while (cal.getTime().compareTo(today) <= 0 && cal.get(Calendar.DAY_OF_MONTH) < cal.getActualMaximum(Calendar.DAY_OF_MONTH)) {
-				String logs = getLogsForDate(cal.getTime());
+				String logs = getLogsForDate(cal.getTime(), user, admin);
 				if (!logs.trim().isEmpty())
 					res.put(cal.getTime(), "Session logs_:_" + logs);
 				cal.add(Calendar.DAY_OF_MONTH, 1);
@@ -367,7 +393,7 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 			int i = 0;
 			while (cal.getTime().compareTo(today) <= 0 && i++ < range) {
 				
-				String logs = getLogsForDate(cal.getTime());
+				String logs = getLogsForDate(cal.getTime(), user, admin);
 				if (!logs.trim().isEmpty())
 					res.put(cal.getTime(), "Session logs_:_" + logs);
 				cal.add(Calendar.DAY_OF_MONTH, 1);
@@ -378,23 +404,22 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 	}
 	
 	
-	private String getLogsForDate(Date date) {
+	private String getLogsForDate(Date date, String user, boolean admin) {
+		
+//		refreshUserLogs(user, date, admin);
+		
+//		getMonthLogs(date, user, admin);
 		
-		// If no month log was retrieved or we need to retrieve logs for a new month
-		if (firstDayOfMonth == null ||
-				!(date.compareTo(firstDayOfMonth) >= 0 && date.compareTo(new Date(firstDayOfMonth.getTime() + MILLIS_IN_MONTH)) < 0)) {
-			getMonthLogs(date);
-		}
 		// Refresh the logs of the day
-		updateTodayLogs(date);
+//		updateTodayLogs(date, user, admin);
 		
 		StringBuilder sb = new StringBuilder();
-		if (sessionLogs.containsKey(date)) {
+		if (sessionLogs.get(user).containsKey(date)) {
 			sb.append("<ul>");
-			for (String bank : sessionLogs.get(date).keySet()) {
+			for (String bank : sessionLogs.get(user).get(date).keySet()) {
 				sb.append("<li>" + bank + "</li>");
 				sb.append("<ul>");
-				List<String> logs = sessionLogs.get(date).get(bank);
+				List<String> logs = sessionLogs.get(user).get(date).get(bank);
 				for (String log : logs) {
 					String[] split = log.split("_:_");
 					if (split.length > 1 && !split[1].trim().isEmpty()) {
@@ -422,44 +447,16 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 		return sb.toString();
 	}
 	
-	/**
-	 * If given date corresponds to today date, returns the corresponding logs.
-	 * 
-	 * @param date
-	 */
-	private void updateTodayLogs(Date date) {
-		Date today = new Date();
-		
-		Calendar calendar = Calendar.getInstance();
-		calendar.setTime(today);
-		calendar.set(Calendar.HOUR_OF_DAY, 0);
-		calendar.set(Calendar.MINUTE, 0);
-		calendar.set(Calendar.SECOND, 0);
-		calendar.set(Calendar.MILLISECOND, 0);
-		
-		if (date.getTime() >= calendar.getTimeInMillis() && date.getTime() < (calendar.getTimeInMillis() + MILLIS_IN_DAY)) {
-			
-			SQLConnection connection = SQLConnectionFactory.getConnection();
-			Statement stat = connection.getStatement();
-			String query = "SELECT startTime,logfile,status FROM session WHERE startTime >= '" + new Timestamp(calendar.getTimeInMillis()) + "' AND startTime < '" +
-					new Timestamp(calendar.getTimeInMillis() + MILLIS_IN_DAY) + "'";
-			
-			sessionLogs.remove(calendar.getTime());
-			
-			ResultSet rs = connection.executeQuery(query, stat);
-			populateSessionLogs(rs);
-			SQLConnectionFactory.closeConnection(stat);
-		}
-	}
 	
 	/**
 	 * Returns logs for the whole month containing the given date.
 	 * 
 	 * @param date
+	 * @param login logs for that login group
+	 * @param is login actually admin
 	 */
-	private void getMonthLogs(Date date) {
+	private void getMonthLogs(Date date, String login, boolean admin) {
 		
-		sessionLogs.clear();
 		Calendar calendar = Calendar.getInstance();
 		calendar.setTime(date);
 		calendar.set(Calendar.DAY_OF_MONTH, 1);
@@ -468,13 +465,23 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 		calendar.set(Calendar.SECOND, 0);
 		calendar.set(Calendar.MILLISECOND, 0);
 		
-		firstDayOfMonth = calendar.getTime();
+		// Was cache was updated recently ?
+		String key = calendar.get(Calendar.YEAR) + "-" + calendar.get(Calendar.MONTH);
+		if (lastCallToSessionLogs.get(key) != null && (System.currentTimeMillis() - lastCallToSessionLogs.get(key)) < CACHE_DURATION) {
+			return;
+		}
+		
+		lastCallToSessionLogs.clear(); // Clear as we only log content for one month
+		
+		sessionLogs.put(login, new HashMap<Date, Map<String,List<String>>>());
+		
+		firstDayOfMonth.put(login, calendar.getTime());
 		
 		SQLConnection connection = SQLConnectionFactory.getConnection();
 		Statement stat = connection.getStatement();
 	
 //		Date today = new Date();
-		long end = firstDayOfMonth.getTime() + MILLIS_IN_MONTH;
+		long end = calendar.getTimeInMillis() + MILLIS_IN_MONTH;
 		/*
 		if ((firstDayOfMonth.getTime() + millisInMonth) > today.getTime()) {
 			calendar.clear();
@@ -487,15 +494,29 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 			end = calendar.getTimeInMillis();
 		}*/
 		
-		String query = "SELECT startTime,logfile,status FROM session WHERE startTime >= '" + new Timestamp(firstDayOfMonth.getTime()) + "' AND startTime < '" +
-				new Timestamp(end) + "'";
+		String query = "";
+		if (admin) {
+			query = "SELECT startTime,logfile,status FROM session WHERE startTime >= '" + new Timestamp(calendar.getTimeInMillis()) + "' AND startTime < '" +
+					new Timestamp(end) + "'";
+		} else {
+			query = "SELECT startTime,logfile,status FROM session WHERE startTime >= '" + new Timestamp(calendar.getTimeInMillis()) + "' AND startTime < '" +
+					new Timestamp(end) + "' AND ref_idupdateBank IN " +
+						"(SELECT idupdateBank FROM updateBank WHERE ref_idconfiguration IN " +
+						"(SELECT idconfiguration FROM configuration WHERE ref_idbank IN " +
+						"(SELECT idbank FROM bank WHERE ref_iduser IN " +
+						"(SELECT ref_iduser FROM bw_user_has_group WHERE ref_idgroup IN " +
+						"(SELECT ref_idgroup FROM bw_user_has_group WHERE ref_iduser=" +
+						"(SELECT iduser FROM bw_user WHERE login='" + login + "'))))))";
+		}
 		
 		ResultSet rs = connection.executeQuery(query, stat);
-		populateSessionLogs(rs);
+		populateSessionLogs(rs, login);
 		SQLConnectionFactory.closeConnection(stat);
+		
+		lastCallToSessionLogs.put(key, System.currentTimeMillis());
 	}
 	
-	private void populateSessionLogs(ResultSet rs) {
+	private void populateSessionLogs(ResultSet rs, String user) {
 		
 		Calendar calendar = Calendar.getInstance();
 		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
@@ -521,20 +542,20 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 					String bankName = log.substring(index, substr2.length());
 	//				int index = log.substring(0, log.lastIndexOf('/'));
 					
-					if (sessionLogs.containsKey(calendar.getTime())) {
-						if (sessionLogs.get(calendar.getTime()).containsKey(bankName)) {
-							sessionLogs.get(calendar.getTime()).get(bankName).add(sdf.format(time) + "_:_" + log + "_:_" + status);
+					if (sessionLogs.get(user).containsKey(calendar.getTime())) {
+						if (sessionLogs.get(user).get(calendar.getTime()).containsKey(bankName)) {
+							sessionLogs.get(user).get(calendar.getTime()).get(bankName).add(sdf.format(time) + "_:_" + log + "_:_" + status);
 						} else {
 							List<String> list = new ArrayList<String>();
 							list.add(sdf.format(time) + "_:_" + log + "_:_" + status);
-							sessionLogs.get(calendar.getTime()).put(bankName, list);
+							sessionLogs.get(user).get(calendar.getTime()).put(bankName, list);
 						}
 					} else {
 						Map<String, List<String>> bankLogs = new HashMap<String, List<String>>();
 						List<String> list = new ArrayList<String>();
 						list.add(sdf.format(time) + "_:_" + log + "_:_" + status);
 						bankLogs.put(bankName, list);
-						sessionLogs.put(calendar.getTime(), bankLogs);
+						sessionLogs.get(user).put(calendar.getTime(), bankLogs);
 					}
 				}
 			}
@@ -542,5 +563,48 @@ public class SchedulingServiceImpl extends RemoteServiceServlet implements Sched
 			log.error(e);
 		}
 	}
+
+
+	@Override
+	public boolean isStarted() {
+		try {
+			return !sched.isInStandbyMode();
+		} catch (SchedulerException e) {
+			e.printStackTrace();
+			return false;
+		}
+	}
+
+
+	@Override
+	public boolean stop() {
+		log.debug("Putting scheduler on stand-by");
+		try {
+			if (!sched.isInStandbyMode()) {
+				sched.standby();
+			}
+			return true;
+		} catch (SchedulerException e) {
+			e.printStackTrace();
+			log.error(e);
+			return false;
+		}
+	}
+
+
+	@Override
+	public boolean start() {
+		log.debug("Resuming scheduler");
+		try {
+			if (sched.isInStandbyMode()) {
+				sched.start();
+			}
+			return true;
+		} catch (SchedulerException e) {
+			e.printStackTrace();
+			log.error(e);
+			return false;
+		}
+	}
 	
 }
