@@ -11,9 +11,13 @@ public class UpdateJob implements Job {
 
 	@Override
 	public void execute(JobExecutionContext context) throws JobExecutionException {
-		
+
 		String banksToUpdate = context.getJobDetail().getDescription();
 		if (!banksToUpdate.isEmpty()) {
+			if(MaintenanceServiceImpl.getMode()) {
+				log.info("Maintenance mode, skipping " + banksToUpdate);
+				return;
+			}
 			log.info("Updating " + banksToUpdate);
 			new BiomajLauncherServiceImpl().startUpdate(banksToUpdate);
 		}
